/*

 * Created on Nov 20, 2004

 *

 * To change the template for this generated file go to

 * Window - Preferences - Java - Code Generation - Code and Comments

 */

package yaes.application.clustering;



import yaes.framework.algorithm.routing.wca.WCAParameters;

import yaes.framework.simulation.DummySimulation;

import yaes.framework.simulation.IContext;

import yaes.framework.simulation.ISimulationCode;

import yaes.framework.simulation.SimulationInputParameters;

import yaes.framework.simulation.SimulationOutputParameters;

import yaes.framework.world.network.sensornetwork.SensorNetwork;

import yaes.ui.visualization.NetworkPainter;

import yaes.ui.visualization.VisualMap;



/**

 * @author Lotzi Boloni

 *

 * To change the template for this generated type comment go to

 * Window - Preferences - Java - Code Generation - Code and Comments

 */

public class Clustering implements ISimulationCode, Constants {



    ISimulationCode clusteringAlgorithmSimulation; // = new WCASimulation();

    ISimulationCode mobilitySimulation; // = new MobilitySimulation();

    ISimulationCode energySimulation; // = new EnergySimulation();

    //ISimulationCode energySimulation = new DummySimulation();

    

    /* (non-Javadoc)

     * @see yaes.framework.simulation.ISimulationCode#setup(yaes.framework.simulation.SimulationInputParameters, yaes.framework.simulation.IContext)

     */

    public void setup(SimulationInputParameters sip, IContext theContext) {

        ClusteringContext context = (ClusteringContext) theContext;

        // the clustering algorithm

        int clustering = sip.getParameterInteger(SIMULATION_CODE_CLUSTERING);

        if (clustering == SC_CLUSTERING_WCA) {

            clusteringAlgorithmSimulation = new WCASimulation();

        } else {

            clusteringAlgorithmSimulation = new DummySimulation();

        }

        // the mobility model

        int mobility = sip.getParameterInteger(SIMULATION_CODE_MOBILITY);

        if (mobility == SC_MOBILITY_RANDOM) {

            mobilitySimulation = new MobilitySimulation();

        } else {

            mobilitySimulation = new DummySimulation();

        };

        // the energy consumption model

        int energymodel = sip.getParameterInteger(SIMULATION_CODE_ENERGY);

        if (energymodel == SC_ENERGY_4THPOWER) {

            energySimulation = new EnergySimulation();

        } else {

            energySimulation = new DummySimulation();

        }

        

        

        // generate the random network

        context.n = new SensorNetwork();

        //context.am.N = sip.getParameterInteger(NODECOUNT);

        double w1 = sip.getParameterDouble(WCA_W1);

        double w2 = sip.getParameterDouble(WCA_W2);

        double w3 = sip.getParameterDouble(WCA_W3);

        double w4 = sip.getParameterDouble(WCA_W4);

        int optimalDegree = sip.getParameterInteger(WCA_OPTIMAL_DEGREE);

        

        // context.wcapar = new WCAParameters(0.7, 0.2, 0.05, 0.05);

        context.wcapar = new WCAParameters(w1, w2, w3, w4, optimalDegree);

        

        context.startTime = 0;

        if (sip.getParameterInteger(VISUALIZE) == VISUALIZE_YES) {            

            context.vn = new VisualMap();

            context.vn.addObject(context.n, new NetworkPainter());

        }

        mobilitySimulation.setup(sip, theContext);

        energySimulation.setup(sip, theContext);

        clusteringAlgorithmSimulation.setup(sip, theContext);

    }





    

    



    /* (non-Javadoc)

     * @see yaes.framework.simulation.ISimulationCode#update(double, yaes.framework.simulation.SimulationInputParameters, yaes.framework.simulation.SimulationOutputParameters, yaes.framework.simulation.IContext)

     */

    public void update(double time, SimulationInputParameters sip,

            SimulationOutputParameters sop, IContext theContext) {

        ClusteringContext context = (ClusteringContext) theContext;

        //context.am.currentTime = (int)time;

        //TextUi.println("Time: " + time);

        //   ... change node coordinate

        mobilitySimulation.update(time, sip, sop, theContext);

        clusteringAlgorithmSimulation.update(time, sip, sop, theContext);        

        energySimulation.update(time, sip, sop, theContext);

        // if any of the nodes failed, then create output for histogram

        // FIXME: this will be part of the measurements of the balanced nature of WCA. 

        /*

        if (context.n.failedNodes.size() != 0) {

            

            TextUi.println("Node energies");

            for (Iterator iter = context.n.nodeInfoList.iterator(); iter.hasNext();) {

                SensorNode element = (SensorNode) iter.next();

                TextUi.print(element.getEnergyModel().getEnergy() + " ");                

            }

            for (Iterator iter = context.n.failedNodes.iterator(); iter.hasNext();) {

                SensorNode element = (SensorNode) iter.next();

                TextUi.print(element.getEnergyModel().getEnergy() + " ");                

            }

            TextUi.println("");

        }

        */

        if (sip.getParameterInteger(VISUALIZE) == VISUALIZE_YES) {

            context.vn.repaint();

            try {

              Thread.sleep(100);

            } catch(InterruptedException iex) {

                

            }

        }

        /*

         if (simulationTime % 5 == 0) {

         vn.save("image"+count+".png");

         count++;

         }

         */

    }



    /* (non-Javadoc)

     * @see yaes.framework.simulation.ISimulationCode#postprocess(yaes.framework.simulation.SimulationInputParameters, yaes.framework.simulation.SimulationOutputParameters, yaes.framework.simulation.IContext)

     */

    public void postprocess(SimulationInputParameters sip,

            SimulationOutputParameters sop, IContext theContext) {

        // compute averages

        /*

         am.avgNoClusterHead = am.acctClusterHead / (Constants.NO_TIMES + 1);

         am.avgCHNeighborChange = ((double) am.noCHChanges)

         / (Constants.NO_TIMES + 1);

         am.avgDominantSetUpdate = ((double) am.dominantSetUpdate)

         / (Constants.NO_TIMES + 1);

         * 

         */

        clusteringAlgorithmSimulation.postprocess(sip, sop, theContext);

    }

}

