/*
 * Created on Mar 25, 2005
 *
 */
package yaes.application.eel6938.classagents.cxbr;

import java.util.HashMap;
import java.awt.Point;
import yaes.application.eel6938.game.objects.Agent;
import yaes.application.eel6938.classagents.CxBRAgent;

/**
 * This class represents the CxBR Engine and Mission Context used for controlling the Agent 
 * @author Viet Trinh
 *
 */
public class CxBR_Engine 
{
	private Agent _agent = null;
	private CxBR_Map _map = null;
	private String _active_context;
	private String _default_context;
	private HashMap<String, CxBR_Context> _contexts;
	private int _timesteps = 0;
	private int _color[] = null;
	private Point _me;
	
	/**
	 * Constructor used to generate the CxBR Engine
	 * @param a The Agents that will be controlled by the CxBR engine
	 */
	public CxBR_Engine(Agent a) 
	{
		this._agent = a;
		this._map = new CxBR_Map();
		this._me = new Point(0, 0);
		
		//Declare Contexts
		this._contexts = new HashMap<String, CxBR_Context>();
		this._contexts.put("ExploreContext", new ExploreContext("ExploreContext", this._agent, this._map));
		this._contexts.put("EatContext", new EatContext("EatContext", this._agent, this._map));
		this._contexts.put("BackTrackContext", new BackTrackContext("BackTrackContext", this._agent, this._map));
		this._contexts.put("AttackContext", new AttackContext("AttackContext", this._agent, this._map));
		this._contexts.put("AvoidContext", new AvoidContext("AvoidContext", this._agent, this._map));
		this._contexts.put("MateContext", new MateContext("MateContext", this._agent, this._map));
		this._contexts.put("FleeContext", new FleeContext("FleeContext", this._agent, this._map));
		this._contexts.put("NearDeathContext", new NearDeathContext("NearDeathContext", this._agent, this._map));
		
		
		//Declare Initial and Default Contexts
		this._active_context = "ExploreContext";
		this._default_context = "ExploreContext";
	}
	
	/**
	 * Use this function to execute a timestep using the CxBR Engine. This will search for the next active context
	 * and execute the context logic contained within the active context
	 *
	 */
	public void execute() 
	{
		//System.out.println("Start Cycle");
		this._timesteps++;
		this._map.explore(this._agent.getSensorDataMap());
		
		//if (this._timesteps % 100 == 0 )
		//	this._map.printMoveTrace();
			
		//Get Next Context - If there is a Context Switch
		try
		{
			if (this._execute_universal_sentinel_rules() == false)
				this._active_context = ((CxBR_Context) this._contexts.get(this._active_context)).executeSentinelRules();
			((CxBR_Context) this._contexts.get(this._active_context)).execute();
			this._color = ((CxBR_Context) this._contexts.get(this._active_context)).getColor();
			//System.out.println(this._agent.getId(this._me) + " - " + this._active_context + " - " + this._timesteps);
		}
		//If the current active context causes an exeception, then use default context
		catch (Exception e)
		{
			this._active_context = this._default_context;
			((CxBR_Context) this._contexts.get(this._active_context)).execute();
			this._color = ((CxBR_Context) this._contexts.get(this._active_context)).getColor();
			//System.out.println(this._agent.getId(this._me) + " - " + this._active_context + " [Exception] - " + this._timesteps);
		}
		//System.out.println("End Cycle");
	}
	
	/**
	 * Use this function to determine if any universal sentinel rule fires. If any fires, all of the logic will be
	 * executed in this function
	 * @return true if the universal sentinel rule fires
	 */
	private boolean _execute_universal_sentinel_rules()
	{
		//Return true is the universal sentinel rule executes and takes priority over transition sentinel rule
		CxBR_Context _temp = ((CxBR_Context) this._contexts.get(this._active_context));
		
		//Universal Final Move Sentinel Rules: if you are about to die, then perform a final move
		if (this._agent.getEnergyLevel() < CxBRAgent.NEAR_DEATH_MIN_ENERGY)
		{
			this._active_context = "NearDeathContext";
			return true;
		}
		
		//Universal Fleeing Sentinel Rules: if you are getting attacked, then flee away
		if (this._active_context != "FleeContext" && _temp.canFlee(this._active_context) != -1)
		{
			this._active_context = "FleeContext";
			return true;
		}
		
		//Universal Matting Sentinel Rules: if the time is right, and you can mate, then mate
		if ((this._timesteps % CxBRAgent.MATE_INTERVAL == 0) && (_temp.canMate()))
		{
			this._active_context = "MateContext";
			return true;
		}
		
		return false;
	}
	
	/**
	 * Use this function to get the contextual color of the agent
	 * @return The color of the current context
	 */
	public int[] getColor()
	{
		//int[] temp = {0, 0, 255};
		//return temp;
		return this._color;
	}
}

