package yaes.application.eel6938.classagents.expertagent;

/* Author: Majid Ali Khan
 * Dated: 04-01-2005
*/
import yaes.application.eel6938.game.Configuration;
import yaes.application.eel6938.game.Constants;

import java.awt.Point;
import java.util.Properties;
/*
 *This class provides an implementation of local map of the agent.
 *It stores the map in a two dimensional array of bytes.
 *
 *Note that it stores a snapshot of current sensor environment.
 *It is not updated with the movement of the agent. I intend to
 *keep it as a historical snap shots of agents. Users can modify it
 *for their own purposes.
 *
 *Map information is contained as follows:
 *(-19,-19) South West Corner  --> (0,0)
 *(-19,19) North West Corner  --> (0,38)
 *(19,-19) South East Corner  --> (38,0)
 *(19,19) North East Corner  --> (38,38)
 
 */
public class LocalMap {

	private byte[][] localMap;
	private int numberOfAgents;
	private int numberOfFood;
	private int numberOfObstacles;
	//private int numberOfOpenSlots;
	//private int numberOfObstacles;

	// Constructor. For a sensor range of say 20, initialize two dimensional
	// array of 39x39
	public LocalMap() {
		localMap = new byte[(Configuration.AgentInitialSensorRange)*2+1][(Configuration.AgentInitialSensorRange)*2+1];
		numberOfAgents = 0;
		numberOfFood = 0;
		numberOfObstacles = 0;
	}
	// Set map information with given type. Also maintain the number
	// of each type of entity (interested in agent or food for now)
	public void setMapInformation(int x, int y, byte type) {
		if (type == Constants.ContentAgent) numberOfAgents++;
		if (type == Constants.ContentFood) numberOfFood++;
		if (type == Constants.ContentObstacle) numberOfObstacles++;
		//System.out.println("x:" + (Configuration.AgentInitialSensorRange-1+x));
		//System.out.println("y:" + (Configuration.AgentInitialSensorRange-1+y));
		localMap[Configuration.AgentInitialSensorRange+x][Configuration.AgentInitialSensorRange+y] = type;
	}
	public void buildMapInformation(Point[] focusPoints, byte type) {
		if (focusPoints == null || focusPoints.length<1) return;
		if (type == Constants.ContentAgent) numberOfAgents += focusPoints.length;
		if (type == Constants.ContentFood) numberOfFood += focusPoints.length;
		if (type == Constants.ContentObstacle) numberOfObstacles += focusPoints.length;
		int x=0, y=0;
		for (int i=0; i<focusPoints.length; i++) {
			x = (int)focusPoints[i].getX();
			y = (int)focusPoints[i].getY();
			//System.out.println("Building Map "+"X:" + x + " Y:"+y);
			localMap[Configuration.AgentInitialSensorRange+x][Configuration.AgentInitialSensorRange+y] = type;	
		}
	}
	public byte getMapInformation(int x, int y) {
		return localMap[Configuration.AgentInitialSensorRange+x][Configuration.AgentInitialSensorRange+y];
	}
	public int getNumberOfAgents() {
		return numberOfAgents;
	}
	public int getNumberofFood() {
		return numberOfFood;
	}
	public int getNumberOfObstacles() {
		return numberOfObstacles;
	}
	public boolean canMove(int direction, int steps) {
		int i=0;
		if (direction == Constants.DirectionNorth) {
			for (i=0; i<steps; i++) {
				if (getMapInformation(0,i) == Constants.ContentObstacle) return false;
			}
			return true;
		}
		if (direction == Constants.DirectionSouth) {
			for (i=0; i<steps; i++) {
				if (getMapInformation(0,-steps) == Constants.ContentObstacle) return false;
			}
			return true;
		}
		if (direction == Constants.DirectionEast) {
			for (i=0; i<steps; i++) {
				if (getMapInformation(steps,0) == Constants.ContentObstacle) return false;
			}
			return true;
		}
		if (direction == Constants.DirectionWest) {
			for (i=0; i<steps; i++) {
				if (getMapInformation(-steps,0) == Constants.ContentObstacle) return false;
			}
			return true;
		}
		return false;
	}
	// This method can be used to determine the number of entities
	// present in a given direction within this local map.
	public int getNumberOfEntites(int direction, int type) {
		int xBegin=0, xEnd=0, yBegin=0, yEnd=0;
		int temp=0;
		int numOfEntities = 0;
		if (direction == Constants.DirectionNorth) {
			xBegin = -(Configuration.AgentInitialSensorRange);
			xEnd = Configuration.AgentInitialSensorRange;
			yBegin = Configuration.AgentInitialSensorRange;
			yEnd = 0;
		} else if (direction == Constants.DirectionSouth) {
			xBegin = -(Configuration.AgentInitialSensorRange);
			xEnd = Configuration.AgentInitialSensorRange;
			yBegin = -1;
			yEnd = -(Configuration.AgentInitialSensorRange);			
		} else if (direction == Constants.DirectionWest) {
			xBegin = -(Configuration.AgentInitialSensorRange);
			xEnd = 0;
			yBegin = Configuration.AgentInitialSensorRange;
			yEnd = - (Configuration.AgentInitialSensorRange);			
		}else if (direction == Constants.DirectionEast) {
			xBegin = 1;
			xEnd = Configuration.AgentInitialSensorRange;
			yBegin = Configuration.AgentInitialSensorRange;
			yEnd = -(Configuration.AgentInitialSensorRange);			
		}
		if (xBegin > xEnd) {
			temp = xBegin;
			xBegin = xEnd;
			xEnd = temp;
		}
		if (yBegin > yEnd) {
			temp = yBegin;
			yBegin = yEnd;
			yEnd = temp;
		}
		for (int i=xBegin; i<=xEnd; i++)
			for (int j=yBegin; j<=yEnd; j++) {
				if (i!= 0 && j!=0)
					if (getMapInformation(i,j) == type) numOfEntities++;
			}
		
		return numOfEntities;
	}
	
	public byte[][] getLocalMap() {
		return localMap;
	}
	public String toString() {
		String returnValue = "LocalMap length:" + localMap.length + "x"+ localMap[0].length + "\r\n";
		if (localMap != null) {
			for (int i=0; i< localMap.length; i++) {
				for (int j=0; j<localMap.length; j++) {
					returnValue += " " + localMap[j][i];
				}
				returnValue += "\r\n";
			}
		}
		return returnValue;
	}
	// Test
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		Properties prop = Configuration.LoadFromPropertiesFile(null,Configuration.AppResourcesPath+Configuration.AppConfigurationPath);
		Configuration.UpdateConfiguation(prop);
		LocalMap map = new LocalMap();
		//System.out.println(map);
		map.setMapInformation(0,0,(byte)Constants.ContentAgent);
		map.setMapInformation(1,1,(byte)Constants.ContentAgent);
		map.setMapInformation(1,2,(byte)Constants.ContentAgent);
		map.setMapInformation(-19,-19,(byte)Constants.ContentAgent);
		map.setMapInformation(-20,-20,(byte)Constants.ContentAgent);
		map.setMapInformation(-18,-19,(byte)Constants.ContentAgent);
		map.setMapInformation(-18,-17,(byte)Constants.ContentAgent);		
		map.setMapInformation(19,-19,(byte)Constants.ContentAgent);
		map.setMapInformation(-19,19,(byte)Constants.ContentAgent);
		map.setMapInformation(19,19,(byte)Constants.ContentAgent);		
		map.setMapInformation(-11,10,(byte)Constants.ContentAgent);
		map.setMapInformation(12,5,(byte)Constants.ContentAgent);
		map.setMapInformation(17,18,(byte)Constants.ContentAgent);
		
		//System.out.println(map);
		//System.out.println(map.getMapInformation(12,5));		
		//System.out.println(map.getLocalMap()[31][24]);
		//System.out.println(map.getLocalMap());
		//System.out.println(map.getNumberOfEntites(Constants.DirectionNorth, Constants.ContentAgent));
		//System.out.println(map.getNumberOfEntites(Constants.DirectionSouth, Constants.ContentAgent));
		//System.out.println(map.getNumberOfEntites(Constants.DirectionEast, Constants.ContentAgent));
		//System.out.println(map.getNumberOfEntites(Constants.DirectionWest, Constants.ContentAgent));
		
		//System.out.println(map.getNumberOfEntites(Constants.DirectionWest, Constants.ContentFood));
	}

}
