/*
 * Ruler.java
 *
 * Created on March 28, 2005, 4:23 PM
 */

package yaes.application.eel6938.classagents.ruleEngine;
import java.util.Random;
import java.util.ArrayList;
import java.awt.Point;
import yaes.application.eel6938.game.Constants;
import yaes.application.eel6938.classagents.RuleBased;

/**
 *
 * @author johann
 */
public class Engine {
    
    KnowledgeBase factBase;
    ArrayList pack;
    Random rnd;
    RuleBased self;
    
    
    /** Creates a new instance of Ruler */
    public Engine(RuleBased a) {
        
        factBase = new KnowledgeBase();
        rnd = new Random();
        pack = new ArrayList();
        self = a;
    }
    
    public Engine(KnowledgeBase k, RuleBased a){
        factBase = k;
        rnd = new Random();
        pack = new ArrayList();
        self = a;
    }
    
    public int[] nextAction(Point[] f, Point[] o, SensedAgent[] a, int[][] m, boolean[] acts){
        
        int i;
        int[] act = null;
        pack.clear();
        factBase.remove("FLOCK");
        
        boolean[] fired = new boolean[factBase.ruleBase.size()];
        factBase.setFacts(f, o, a, m, acts);
        ArrayList activeRules;
        Rule temp;
        for(i=0;i<factBase.ruleBase.size();i++){
            temp=(Rule)factBase.ruleBase.get(i);
            if(!fired[i]){
                if(evaluateRule(temp)){
                    if(temp.consequentString!=null){
                        factBase.add(temp.consequentString);
                    }else if(temp.consequent[0]==RC.MOVE||temp.consequent[0]==RC.ATTACK||temp.consequent[0]==RC.FLEE||temp.consequent[0]==RC.EAT||temp.consequent[0]==RC.MATE){
                        return temp.consequent;
                    }
                    fired[i]=true;
                }
            }
        }
        
        return act;
    }
    
    public void addKnowledge(KnowledgeBase k){
        factBase = k;
    }
    
    private boolean evaluateRule(Rule r){
        int i;
        boolean agent, friend;
        for(i=0;i<r.antecedents.length;i++){
            if(!factBase.containFact(r.antecedents[i])){
                return false;
            }
        }
        if(r.antecedentsString!=null){
            for(i=0;i<r.antecedentsString.length;i++){
                if(!factBase.containFact(r.antecedentsString[i])){
                    return false;
                }
            }
        }
        switch(r.consequent[0]){
            case RC.MOVE:
                switch (r.consequent[4]){
                    case RC.TOFOOD:
                        return moveTowardFood(r);
                    case RC.AVOIDENEMY:
                        
                        return avoidEnemy(r);
                    case RC.TOENEMY:
                        return moveTowardEnemy(r);
                    case RC.FLOCK:
                        return flockUp(r);
                }
            case RC.FLOCK:
                flock(r);
                return true;
            case RC.EAT:
                return true;
            case RC.ATTACK:
                return attack(r);
            case RC.AGGRESSIVE:
                aggression(r);
                return true;
            case RC.FLEE:
                return flee(r);
            case RC.MATE:
                return mateNow();
            case RC.DOUBLEAGENT:
                return doubleAgent(r);
        }
        return false;
    }
    
    private boolean attack(Rule r){
        int i;
        
        for(i=1;i<factBase.agents.length;i++){
            if(factBase.agents[i].specie.compareTo(self.realName)!=0 && !factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id))){
                if(factBase.agents[i].point.x==1&&factBase.agents[i].point.y==0){
                    r.consequent[1]=(Constants.DirectionEast);
                    return true;
                }
                if(factBase.agents[i].point.x==0&&factBase.agents[i].point.y==1 ){
                    r.consequent[1]=(Constants.DirectionNorth);
                    return true;
                }
                if(factBase.agents[i].point.x==0&&factBase.agents[i].point.y==-1){
                    r.consequent[1]=(Constants.DirectionSouth);
                    return true;
                }
                if(factBase.agents[i].point.x==-1&&factBase.agents[i].point.y==0){
                    r.consequent[1]=(Constants.DirectionWest);
                    return true;
                }
            }
        }
        return false;
    }
    
    private boolean mateNow(){
        if(factBase.agents[0].energy<2000){
            return true;
        }else{
            return false;
        }
    }
    
    private boolean moveTowardFood(Rule r){
        int i;
        int j;
        if(factBase.containFact("FLOCK") && !factBase.containFact("GREEDY")){
            if(factBase.availableActions[RC.EAT]){
                return false;
            }
            for(j=0;j<factBase.food.length;j++){
                for(i=0;i<pack.size();i++){
                    if(distance(factBase.agents[0].point,factBase.food[j])>distance(((SensedAgent)pack.get(i)).point ,factBase.food[j])){
                        if(rnd.nextInt(5)!=3){
                            break;
                        }
                    }
                }
                if(i==pack.size()){
                    r.consequent[1]=factBase.food[j].x;
                    r.consequent[2]=factBase.food[j].y;
                    return true;
                }
            }
        }else{
            if(factBase.availableActions[RC.EAT]){
                return false;
            }
            r.consequent[1]=factBase.food[0].x;
            r.consequent[2]=factBase.food[0].y;
            return true;
        }
        return false;
    }
    
    private boolean flockUp(Rule r){
        int self= factBase.agents[0].id;
        int i;
        SensedAgent ally;
        for(i=0;i<pack.size();i++){
            ally = (SensedAgent)pack.get(i);
            if(ally.id<self){
                r.consequent[1]=ally.point.x+5+(self-ally.id)%12;
                r.consequent[2]=ally.point.y+5+(self-ally.id)%12;
                return true;
            }
        }
        return false;
    }
    
    private void flock(Rule r){
        int self= factBase.agents[0].id;
        int i;
        for(i=1;i<factBase.agents.length;i++){
            if(factBase.agents[i].specie.compareTo(this.self.realName)==0){
                pack.add(factBase.agents[i]);
                factBase.add("FLOCK");
                factBase.add("ALLY"+String.valueOf(factBase.agents[i].id));
            }else if(factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id))){
                pack.add(factBase.agents[i]);
                factBase.add("FLOCK");
            }
        }
        if(pack.size()==0){
            factBase.remove("FLOCK");
        }
    }
    
    private boolean flee(Rule r){
        int i;
        
        if(factBase.containFact("DOUBLEAGENT")){
            self.disguise(self.realName);
            return false;
        }
        for(i=1;i<factBase.agents.length;i++){
            if(factBase.agents[i].specie.compareTo(self.realName)!=0 && !factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id))){
                if(factBase.agents[i].point.x==1&&factBase.agents[i].point.y==0){
                    r.consequent[1]=(Constants.DirectionWest);
                    return true;
                }
                if(factBase.agents[i].point.x==0&&factBase.agents[i].point.y==1){
                    r.consequent[1]=(Constants.DirectionSouth);
                    return true;
                }
                if(factBase.agents[i].point.x==0&&factBase.agents[i].point.y==-1){
                    r.consequent[1]=(Constants.DirectionNorth);
                    return true;
                }
                if(factBase.agents[i].point.x==-1&&factBase.agents[i].point.y==0){
                    r.consequent[1]=(Constants.DirectionEast);
                    return true;
                }
            }
        }
        return false;
    }
    
    private boolean moveTowardEnemy(Rule r){
        int i;
        if(factBase.availableActions[RC.ATTACK]){
            return false;
        }
        for(i=1;i<factBase.agents.length;i++){
            //System.out.println(factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id)));
            if (self.realName.compareTo(factBase.agents[i].specie)!=0 && !factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id))){
                r.consequent[1]=factBase.agents[i].point.x;
                r.consequent[2]=factBase.agents[i].point.y;
                return true;
            }
        }
        return false;
    }
    
    private boolean avoidEnemy(Rule r){
        int i;
        for(i=1;i<factBase.agents.length;i++){
            if (self.realName.compareTo(factBase.agents[i].specie)!=0 && !factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id))){
                r.consequent[1]=-factBase.agents[i].point.x*5;
                r.consequent[2]=-factBase.agents[i].point.y*5;
                return true;
            }
        }
        return false;
    }
    
    private void aggression(Rule r){
        int i;
        
        if(r.consequent[2]==0){
            factBase.add("WUSSY");
            factBase.remove("AGGRESSIVE");
            return;
        }
        float ratio = (float)r.consequent[1]/(float)r.consequent[2];
        float difference;
        SensedAgent guy;
        for(i=1;i<factBase.agents.length;i++){
            guy = factBase.agents[i];
            if(guy.specie.compareTo(self.realName)!=0 && !factBase.containFact("ALLY"+String.valueOf(factBase.agents[i].id))){
                if(guy.energy<=0){
                    factBase.remove("AGGRESSIVE");
                    factBase.add("WUSSY");
                    return;
                }
                difference = (float)factBase.agents[0].energy/(float)guy.energy;
                if(difference>ratio){
                    
                    factBase.add("AGGRESSIVE");
                    factBase.remove("WUSSY");
                    return;
                }else{
                    factBase.remove("AGGRESSIVE");
                    factBase.add("WUSSY");
                    return;
                }
            }
        }
    }
    
    private boolean doubleAgent(Rule r){
        int i;

        for(i=1;i<factBase.agents.length;i++){
            if(factBase.agents[i].specie.compareTo(self.realName)!=0 && !factBase.containFact("ALLY"+factBase.agents[i].id)){
                factBase.add("DOUBLEAGENT");
                self.disguise(factBase.agents[i].specie);
                return true;
            }
        }
        factBase.remove("DOUBLEAGENT");
        self.disguise(self.realName);
        return false;
    }
    
    private double distance(Point a, Point b){
        int x = a.x-b.x;
        int y = a.y-b.y;
        return (x*x+y*y);
    }
}
    
