/*
 * Created on Jan 26, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.eel6938.game.objects;

import java.awt.Color;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import yaes.framework.world.map.IMap;
import yaes.framework.world.map.Location;

/**
 * @author Linus Luotsinen
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Map implements IMap {
	private static String m_EnergyObjectName = "Energy";
	private static String m_ObstacleObjectName = "Obstacle";
	private static String m_AgentObjectName = "Agent";
	
	private int m_Width;
	private int m_Height;
	
	private ArrayList<Energy> m_EnergyList;
	private ArrayList<Agent> m_AgentList;
	private Agent[][] m_AgentMap;
	private Energy[][] m_EnergyMap;
	private boolean[][] m_ObstacleMap;
	
	private BufferedImage m_Image = null;
	
	/**
	 * 
	 */
	public Map(int width, int height) {
		super();
		// TODO Auto-generated constructor stub
		m_Width = width;
		m_Height = height;
		m_EnergyMap = new Energy[m_Width][m_Height];
		m_ObstacleMap = new boolean[m_Width][m_Height];
		m_AgentMap = new Agent[m_Width][m_Height];
		m_AgentList = null;
		m_EnergyList = null;
		clearMap();
	}
	
	void clearMap()
	{
		clearObstacleMap();
		clearEnergyMap();
		clearAgentMap();
	}
	
	BufferedImage getImage()
	{
		return m_Image;
	}
	
	void clearAgentMap()
	{
		for(int x=0;x<m_Width;x++)
		{
			for(int y=0;y<m_Height;y++)
			{
				m_AgentMap[x][y] = null;
			}
		}
	}
	
	void clearObstacleMap()
	{
		for(int x=0;x<m_Width;x++)
		{
			for(int y=0;y<m_Height;y++)
			{
				m_ObstacleMap[x][y] = false;
			}
		}
	}
	
	void clearEnergyMap()
	{
		for(int x=0;x<m_Width;x++)
		{
			for(int y=0;y<m_Height;y++)
			{
				m_EnergyMap[x][y] = null;
			}
		}
	}
	
	public void refreshMaps()
	{
		refreshAgentMap();
	}
	
	private void refreshAgentMap()
	{
		Agent[] agentsArray = new Agent[m_AgentList.size()];
		m_AgentList.toArray(agentsArray);
		
		clearAgentMap();
		for(int i=0;i<agentsArray.length; i++)
		{
			if(agentsArray[i].isAlive() &&agentsArray[i].isEnabled()  )
			{
				setPropertyAt(m_AgentObjectName, agentsArray[i].getX(), agentsArray[i].getY(), agentsArray[i]);
			}
			
		}
	}
	
	public ArrayList<Agent> getAgentList()
	{
		return m_AgentList;
	}
	
	public ArrayList<Energy> getEnergyList()
	{
		return m_EnergyList;
	}
	
	public void populateAgentList(ArrayList<Agent> agents)
	{
		m_AgentList = agents;
	}
	
	public void populateObstacleMap(String imageFileName)
	{
		try 
		{
			BufferedImage baseImage = ImageIO.read(new File(imageFileName));
            Image magnifiedImage = baseImage.getScaledInstance( m_Width, m_Height, BufferedImage.SCALE_SMOOTH);
			BufferedImage scaledImage = new BufferedImage ( m_Width,	m_Height, BufferedImage.TYPE_INT_BGR);
			scaledImage.createGraphics().drawImage(magnifiedImage, 0, 0, null /* observer */ );

			ColorModel cm = scaledImage.getColorModel();
            int imageWidth = scaledImage.getWidth();
            int imageHeight = scaledImage.getHeight();
            
            //write out the BufferedImage as a JPEG
			//ImageIO.write( scaledImage,"JPEG" ,  new File("text.jpg"));
            //System.out.println(imageWidth + "x" + imageHeight);
            
            int obstacleCount = 0;
            for (int x = 0; x != imageWidth; x++)
            {
                for (int y = 0; y != imageHeight; y++) 
                {
                    int pixel = scaledImage.getRGB(x, y);
                    Color c = new Color(pixel);
    				int red = c.getRed();
    				int blue = c.getBlue();
    				int green = c.getGreen();

    				//Obstacle pattern 
                    if ((red < 5) && (green < 5) && (blue < 5)) 
                    {
                    	setPropertyAt(m_ObstacleObjectName, x, y, new Boolean(true));
                    }
                }
            }
            m_Image = scaledImage;
            //System.out.println(obstacleCount);
        } 
            catch (IOException e) {

            e.printStackTrace();
        }
	}
	
	public List<Energy> populateEnergyMap(ArrayList<Energy> agents, int instances)
	{
		if(m_EnergyList == null)
		{
			m_EnergyList = agents;
		}
		
		ArrayList<Energy> newUnits = new ArrayList<Energy>();
		
		int maxFailureCount = 100;
		int failureCount = 0;
		for(int i=0;i<instances;i++)
		{
			int x = (int) (Math.random() * m_Width);
			int y = (int) (Math.random() * m_Height);
			//Only add if position is open
			if(	isOpen(x, y) )
			{
				Energy energyUnit = new Energy();
				energyUnit.setPosition(x, y);
				setPropertyAt(m_EnergyObjectName, x, y, energyUnit);
				failureCount = 0;
				m_EnergyList.add(energyUnit);
				newUnits.add(energyUnit);
			}
			else
			{
				failureCount++;
				i--;
				//System.out.println("Unable to populate map after " + failureCount + " attempts. Trying again...");
			}
			
			if(failureCount > maxFailureCount)
			{
				System.err.println("Unable to populate map with energy unit after " + failureCount + " attempts. Aborting population procedure!");
				break;
			}
		}
		return newUnits;
	}
	
	public boolean isOpen(int x, int y)
	{
		boolean ret = false;
		if(		getPropertyAt("Agent" , x, y) == null &&
				getPropertyAt("Energy" , x, y) == null && 
				getPropertyAtAsBoolean("Obstacle", new Location(x,y)) == false &&
				x >= this.getXLow() && x < getXHigh() &&
				y >= this.getYLow() && y < getYHigh())
		{
			ret = true;
		}
		return ret;
	}
	
	
	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getPropertyAt(java.lang.String, double, double)
	 */
	public void removePropertyAt(String propertyName, double x, double y) { 		
		Object ret = null;
		if(x>=0 && x<(m_Width) && y>=0 && y<(m_Height))
		{
			//Check for Energy Properties
			if(propertyName.compareTo(m_EnergyObjectName) == 0)
			{
				m_EnergyMap[(int)x][(int)y] = null;
			}
			
			if(propertyName.compareTo(m_ObstacleObjectName) == 0)
			{
				m_ObstacleMap[(int)x][(int)y] = false;
			}
			
			if(propertyName.compareTo(m_AgentObjectName) == 0)
			{
				m_AgentMap[(int)x][(int)y] = null;
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getPropertyAt(java.lang.String, double, double)
	 */
	public Object getPropertyAt(String propertyName, double x, double y) { 		
		Object ret = null;
		if(x>=0 && x<(m_Width) && y>=0 && y<(m_Height))
		{
			//Check for Energy Properties
			if(propertyName.compareTo(m_EnergyObjectName) == 0)
			{
				ret = m_EnergyMap[(int)x][(int)y];
			}
			
			if(propertyName.compareTo(m_ObstacleObjectName) == 0)
			{
				ret = new Boolean(m_ObstacleMap[(int)x][(int)y]);
			}
			
			if(propertyName.compareTo(m_AgentObjectName) == 0)
			{
				ret = m_AgentMap[(int)x][(int)y];
			}
		}
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getPropertyAtAsBoolean(java.lang.String, yaes.framework.world.map.Location)
	 */
	public boolean getPropertyAtAsBoolean(String propertyName, Location location) {
		
		boolean ret = false;
		if(location.getX()>=0 && location.getX()<(m_Width) && location.getY()>=0 && location.getY()<(m_Height))
		{
			if(propertyName.compareTo(m_EnergyObjectName) == 0)
			{
				if(m_EnergyMap[(int)location.getX()][(int)location.getY()] != null)
				{
					ret = true;
				}
			}
			
			if(propertyName.compareTo(m_ObstacleObjectName) == 0)
			{
				ret = m_ObstacleMap[(int)location.getX()][(int)location.getY()];
			}
			
			
			if(propertyName.compareTo(m_AgentObjectName) == 0)
			{
				if(m_AgentMap[(int)location.getX()][(int)location.getY()] != null)
				{
					ret = true;
				}
			}
		}
		return ret;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#setPropertyAt(java.lang.String, double, double, java.lang.Object)
	 */
	public void setPropertyAt(String propertyName, double x, double y,
			Object value) {
		if(x>=0 && x<(m_Width) && y>=0 && y<(m_Height))
		{
			if(propertyName.compareTo(m_EnergyObjectName) == 0)
			{
				m_EnergyMap[(int)x][(int)y] = (Energy)value;
			}
			
			if(propertyName.compareTo(m_ObstacleObjectName) == 0)
			{
				Boolean v = (Boolean)value;
				m_ObstacleMap[(int)x][(int)y] = v.booleanValue();
			}
			
			if(propertyName.compareTo(m_AgentObjectName) == 0)
			{
				m_AgentMap[(int)x][(int)y] = (Agent)value;
			}
		}
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getPropertyClass(java.lang.String)
	 */
	public Class getPropertyClass(String propertyName) {
		
		Class ret = null;
		if(propertyName.compareTo(m_EnergyObjectName) == 0)
		{
			ret = Energy.class.getClass();
		}
		
		if(propertyName.compareTo(m_ObstacleObjectName) == 0)
		{
			ret = Boolean.class.getClass();
		}
		
		if(propertyName.compareTo(m_AgentObjectName) == 0)
		{
			ret = Agent.class.getClass();
		}
		
		return ret;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getProperties()
	 */
	public List<Energy> getProperties() {
		List<Energy> ret = new ArrayList<Energy>();
		for(int x=0;x<m_Width;x++)
		{
			for(int y=0; y<m_Height; y++)
			{
				if(m_EnergyMap[x][y] != null)
				{
					ret.add(m_EnergyMap[x][y]);
				}
			}
		}
		return ret;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getXLow()
	 */
	public double getXLow() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getXHigh()
	 */
	public double getXHigh() {
		// TODO Auto-generated method stub
		return m_Width;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getYLow()
	 */
	public double getYLow() {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getYHigh()
	 */
	public double getYHigh() {
		// TODO Auto-generated method stub
		return m_Height;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getXPreferredStep()
	 */
	public double getXPreferredStep() {
		// TODO Auto-generated method stub
		return 1;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getYPreferredStep()
	 */
	public double getYPreferredStep() {
		// TODO Auto-generated method stub
		return 1;
	}

	/* (non-Javadoc)
	 * @see yaes.framework.world.map.IMap#getMapName()
	 */
	public String getMapName() {
		// TODO Auto-generated method stub
		return null;
	}

}
