/*
 * Created on Nov 10, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package yaes.application.empiricalpathplanner.testharness;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import yaes.application.empiricalpathplanner.IDrawable;
import yaes.application.empiricalpathplanner.Path;
import yaes.application.empiricalpathplanner.World;
import yaes.application.empiricalpathplanner.ga.Chromosome;
import yaes.application.empiricalpathplanner.ga.IPathPlannerListener;
import yaes.application.empiricalpathplanner.ga.OperatorFactory;
import yaes.application.empiricalpathplanner.ga.PathPlanner;
import yaes.application.empiricalpathplanner.spline.BSpline;
import yaes.application.empiricalpathplanner.spline.XYZ;
import yaes.framework.algorithm.geneticalgorithm.ICrossover;
import yaes.framework.algorithm.geneticalgorithm.IMutation;

/**
 * @author Linus Luotsinen
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class TestHarness 
	implements IPathPlannerListener,IDrawable
{
	private Experiments experiments;
	private World world;
	private PathPlanner pathPlanner;
	private boolean drawChromosomes = true;
	private final String dataDir;
	private FileWriter out;	
	public TestHarness(World world, String configFile, String dataDir)
	{
		System.out.println("Initializing test harness...");
		experiments = new Experiments(configFile);
		this.world = world;
		this.pathPlanner = world.getPathPlanner();
		this.dataDir = dataDir;
		pathPlanner.addPathPlannerListener(this);
	}

	public void Start()
	{
		Experiment[] e = experiments.getExperiments();
		//Loop the experiments
		for(int i=0; i<e.length; i++)
		{
			try 
			{
				String tmpDataDir = dataDir + e[i].getTitle() + "/";
				System.out.println(tmpDataDir);
				File f = new File(tmpDataDir);
				f.mkdirs();
				
				world.loadImage(e[i].getMap());
				pathPlanner.SetCrossoverRate(e[i].getCrossoverRate());
				pathPlanner.SetMutationRate(e[i].getMutationRate());
				pathPlanner.setConvergenceCriteria(e[i].getConvergenceCriteria());
				yaes.application.empiricalpathplanner.Path start = new yaes.application.empiricalpathplanner.Path(3,50);
				Point [] startGoalpoints = e[i].getStartGoalPath().getPoints();
				for(int j=0; j<startGoalpoints.length;j++)
				{
					start.addControlPoint(new XYZ(startGoalpoints[j].getX(),startGoalpoints[j].getY(),startGoalpoints[j].getZ()),false);
				}
				start.update();
				world.setStartGoalPath(start);
				
				Vector knownChromosomes = new Vector();
				yaes.application.empiricalpathplanner.testharness.Path[] knownPaths = e[i].getKnownPaths().getPaths();		
				for(int j=0;j<knownPaths.length;j++)
				{
					yaes.application.empiricalpathplanner.Path known = new yaes.application.empiricalpathplanner.Path(3,50);
					Point[] knownPoints = knownPaths[j].getPoints();
					
					for(int k=0;k<knownPoints.length;k++)
					{
						known.addControlPoint(new XYZ(knownPoints[k].getX(),knownPoints[k].getY(),knownPoints[k].getZ()),false);
					}
					known.update();
					knownChromosomes.add(known);
				}
				world.setKnownPaths(knownChromosomes);
				
				
				//Set up the operators for this experiment
				Operator[] operators = e[i].getOperators();
				ArrayList crossoverOperators = new ArrayList();
				ArrayList mutationOperators = new ArrayList();
				for(int operatorCount=0; operatorCount< operators.length; operatorCount++)
				{
					Operator currentOp = operators[operatorCount];
					if(currentOp.getType().equalsIgnoreCase("Crossover"))
					{
						crossoverOperators.add(OperatorFactory.CreateCrossoverOperator(currentOp.getId(),pathPlanner));
					}
					
					if(currentOp.getType().equalsIgnoreCase("Mutation"))
					{
						mutationOperators.add(OperatorFactory.CreateMutationOperator(currentOp.getId(), currentOp.getArgs()));
					}
				}
				ICrossover[] c = new ICrossover[crossoverOperators.size()];
				crossoverOperators.toArray(c);
				IMutation[] m = new IMutation[mutationOperators.size()];
				mutationOperators.toArray(m);
				
				if(c.length > 0 && m.length > 0)
				{
					pathPlanner.setOperators(c,m);
				}
				else
				{
					pathPlanner.setOperators(null,null);
				}
				
				
				for(int experimentCount=0; experimentCount < e[i].getExperiments(); experimentCount++)
				{
					File outputFile = new File(tmpDataDir + e[i].getTitle() + "_" + experimentCount + "_data.txt");
					out = new FileWriter(outputFile);
					
					System.out.println(e[i]);
					GenerateChromosomes(e[i].getChromosomes(), e[i].getChromosomeLength(), e[i].isChromosomesRandom(), e[i].isChromosomesFilter());
					
					if(world.getStartGoalPath() == null)
					{
						System.out.println("No Start-Goal Path was created. Aborting!");
						return;
					}
					pathPlanner.evolveBatch(e[i].getEpochs());
					
					drawChromosomes = true;
					world.saveImage(tmpDataDir + e[i].getTitle() + "_" + experimentCount + "_chrom.png", this);	
					drawChromosomes = false;
					world.saveImage(tmpDataDir + e[i].getTitle() + "_" + experimentCount + "_noChrom.png", this);	
					out.close();
				}
			} 
			catch (IOException e1) 
			{
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}

	
	public void GenerateChromosomes(int chromosomes, int chomosomeLength,  boolean randomLengthChomosomes, boolean filterInadmissibleChromosomes)
	{
		System.out.print("Creating random chromosomes... ");
		world.getPathPlanner().getChromosomes().clear();	//Clear old chromosomes
		for(int i = 0; i< chromosomes; i++)
		{
			yaes.application.empiricalpathplanner.Path p;
			int points;
			//Random number of chrmosome lenght
			if(randomLengthChomosomes)
			{
				points = Math.round( (int)(Math.random() * chomosomeLength));
				if(points < 3)
				{
					points = 3;
				}
			}
			else
			{
				points = chomosomeLength;
			}
			p = yaes.application.empiricalpathplanner.Path.GenerateRandomPath(points,3,50,world.getHeight(),world.getWidth(),50,40);
			
			if(isInAdmissible(p.getCurve()) && filterInadmissibleChromosomes)
			{
				//System.out.println("InAdmissible curve detected!");
				i--;
			}
			else
			{
				//System.out.println("Ok curve detected!");
				Chromosome chrom1 = new Chromosome(p);
				world.getPathPlanner().getChromosomes().add(chrom1);
			}
		}
		System.out.println(world.getPathPlanner().getChromosomes().size() + " random chromosomes in list!");
	}
	
	public boolean isInAdmissible(XYZ[] curve)
	{
		boolean inAdmissible = false; //Curve is ok
		
		if(curve != null)
		{
			for(int i=0; i<curve.length-1;i++)	 
			{
				//Get the first and second point
				XYZ[] inpArr = new XYZ[2];
				inpArr[0] = curve[i];
				inpArr[1] = curve[i+1];
				int tmpDist = (int)Math.round(XYZ.distance(curve[i],curve[i+1]));
				
				if(tmpDist > 1)
				{
					XYZ[] tmpCurve = BSpline.createSpline(2,tmpDist,inpArr);
					
					for(int j=0;j<tmpCurve.length;j++)
					{
						if(world.classifyTerrain((int)Math.round(tmpCurve[j].x),(int)Math.round(tmpCurve[j].y)) == World.MOUNTAIN)
						{
							return true; //Curve is not ok!
						}
					}
				}
			}
		}
		return inAdmissible;
	}
	
	/* (non-Javadoc)
	 * @see yaes.application.empiricalpathplanner.ga.IPathPlannerListener#OnEpochComplete(int, int, int)
	 */
	public void OnEpochComplete(int epoch, int mutations, int crossovers) {
		System.out.print(".");
		try {
			Chromosome bestPath = (Chromosome)pathPlanner.getChromosomes().get(0);
			Path currentPath = mergeStartGoalWithPath(bestPath.getChromosomePath());
			double curveLenCost = world.getPathPlanner().pathLengthCost(currentPath);
			double curveCtrlPtsCost = world.getPathPlanner().controlPointDistanceCost(currentPath.getControlPointArray());
			double curveTerrainCost = world.getPathPlanner().terrainCost(currentPath.getCurve());
			double curveCost = curveLenCost + curveCtrlPtsCost + curveTerrainCost;
			double curveLen = currentPath.getPathLenght();
			int ctrlPoints = bestPath.getChromosomePath().getControlPoints().size();
			out.write(epoch + "\t" + curveCost  + "\t" + curveLenCost + "\t" + curveCtrlPtsCost + "\t" + curveTerrainCost + "\t" + curveLen + "\t" + ctrlPoints + "\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see yaes.application.empiricalpathplanner.ga.IPathPlannerListener#OnEvolveComplete()
	 */
	public void OnEvolveComplete() {
		System.out.println("");
		System.out.println("Training completed!");
	}
	
	/* (non-Javadoc)
	 * @see yaes.application.empiricalpathplanner.IDrawable#draw(java.awt.Graphics)
	 */
	public void draw(Graphics g) {
		//Draw image map
		Graphics2D g2d = (Graphics2D) g;
		g2d.drawImage(world.getImage(),null,null);
		
		//Paint world path library lines
		Iterator iter = world.getKnownPaths().iterator();
		while(iter.hasNext())
		{
			Path tmpPath  = (Path) iter.next();
			g.setColor(Color.ORANGE);
			
			if(tmpPath.getCurve() != null)
			{
				for(int i=0; i<tmpPath.getCurve().length-1;i++)
				{
					Stroke stroke = new BasicStroke(8,BasicStroke.JOIN_ROUND, 
													BasicStroke.JOIN_ROUND, 0,
													new float[] { 1, 1 }, 0);
					g2d.setStroke(stroke);

					g.drawLine((int)Math.round(tmpPath.getCurve()[i].x),(int)Math.round(tmpPath.getCurve()[i].y),(int)Math.round(tmpPath.getCurve()[i+1].x),(int)Math.round(tmpPath.getCurve()[i+1].y));
				
					//Color oldColor = g.getColor();
					//g.setColor(Color.LIGHT_GRAY);
					//g.drawString(tmpPath.toString(), (int)Math.round(tmpPath.getControlPointArray()[0].x), (int)Math.round(tmpPath.getControlPointArray()[0].y));
					//g.setColor(oldColor);
				}
			}
		}
		
		
		if(drawChromosomes)
		{
			//Paint Chromosomes
			g.setColor(Color.cyan);
			iter = world.getPathPlanner().getChromosomes().iterator();
			while(iter.hasNext())
			{
				Chromosome tmpChrom  = (Chromosome) iter.next();		
				if(tmpChrom.getChromosomePath().getCurve() != null)
				{
					Path tmpPath = mergeStartGoalWithPath(tmpChrom.getChromosomePath());
					for(int i=0; i<tmpPath.getCurve().length-1;i++)
					{
						Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
														BasicStroke.JOIN_ROUND, 0,
														new float[] { 1, 1 }, 0);
						g2d.setStroke(stroke);
	
						g.drawLine((int)Math.round(tmpPath.getCurve()[i].x),(int)Math.round(tmpPath.getCurve()[i].y),(int)Math.round(tmpPath.getCurve()[i+1].x),(int)Math.round(tmpPath.getCurve()[i+1].y));
					}
				}
			}
		}
		
		//Paint start goal path
		/*g.setColor(Color.YELLOW);
		if(world.getStartGoalPath() != null && world.getStartGoalPath().getCurve() != null)
		{
			Path tmpPath = world.getStartGoalPath();
			for(int i=0; i<tmpPath.getCurve().length-1;i++)
			{
				Stroke stroke = new BasicStroke(2,BasicStroke.JOIN_ROUND, 
												BasicStroke.JOIN_ROUND, 0,
												new float[] { 1, 1 }, 0);
				g2d.setStroke(stroke);
				g.drawLine((int)Math.round(tmpPath.getCurve()[i].x),(int)Math.round(tmpPath.getCurve()[i].y),(int)Math.round(tmpPath.getCurve()[i+1].x),(int)Math.round(tmpPath.getCurve()[i+1].y));
			}
			Color oldColor = g.getColor();
			g.setColor(Color.LIGHT_GRAY);
			g.drawString("Start-Goal", (int)Math.round(tmpPath.getControlPointArray()[0].x), (int)Math.round(tmpPath.getControlPointArray()[0].y));
			g.setColor(oldColor);
		}*/
		
		//	Paint evolved world path library lines based on start and goal path
		if(world.getStartGoalPath() != null)
		{	
			g.setColor(Color.WHITE);
			iter = world.getPathPlanner().getChromosomes().iterator();
			while(iter.hasNext())
			{
				Chromosome tmpChrom  = (Chromosome) iter.next();		
				Path compoundPath = new Path(3, 50);	//The goal start path (the two first points and the last point) + choromosome path
				Vector startGoalPoints = world.getStartGoalPath().getControlPoints();
				
				//Add first and second point
				compoundPath.addControlPoint((XYZ)startGoalPoints.firstElement(), false);
				compoundPath.addControlPoint((XYZ)startGoalPoints.elementAt(1), false);
				//Add intermediate points
				Vector otherPoints = tmpChrom.getChromosomePath().getControlPoints();
				Iterator iter2 = otherPoints.iterator();
				while(iter2.hasNext())
				{
					XYZ p = (XYZ)iter2.next();
					compoundPath.addControlPoint(p, false);
				}	
				//Add goal point from startgoal & update curve!
				compoundPath.addControlPoint((XYZ)startGoalPoints.lastElement());
				
				if(compoundPath.getCurve() != null)
				{
					for(int i=0; i<compoundPath.getCurve().length-1;i++)
					{
						Stroke stroke = new BasicStroke(10,BasicStroke.JOIN_ROUND, 
														BasicStroke.JOIN_ROUND, 0,
														new float[] { 1, 1 }, 0);
						g2d.setStroke(stroke);
	
						g.drawLine((int)Math.round(compoundPath.getCurve()[i].x),(int)Math.round(compoundPath.getCurve()[i].y),(int)Math.round(compoundPath.getCurve()[i+1].x),(int)Math.round(compoundPath.getCurve()[i+1].y));
					}
				}
				break;  //Now only the best is shown
			}
		}
	}
	
	
	public Path mergeStartGoalWithPath(Path path)
	{
		Path startGoal = world.getStartGoalPath();
		// TODO Auto-generated method stub
		Path compoundPath = new Path(3, 50);	//The goal start path (the two first points and the last point) + choromosome path

		Vector startGoalPoints = startGoal.getControlPoints();
		
		//Add first and second point
		compoundPath.addControlPoint((XYZ)startGoalPoints.firstElement(), false);
		compoundPath.addControlPoint((XYZ)startGoalPoints.elementAt(1), false);
		
		//Add intermediate points
		Vector otherPoints = path.getControlPoints();
		Iterator iter = otherPoints.iterator();
		while(iter.hasNext())
		{
			XYZ p = (XYZ)iter.next();
			compoundPath.addControlPoint(p, false);
		}	
		//Add goal point from startgoal & update curve!
		compoundPath.addControlPoint((XYZ)startGoalPoints.lastElement());
		return compoundPath;
	}
}
