package yaes.application.gridbrokerage;



import yaes.ui.plot.PlotLineDescription;

import yaes.framework.simulation.SimulationHelper;

import java.util.ArrayList;

import yaes.framework.simulation.Simulation;

import yaes.framework.simulation.SimulationInputParameters;

import yaes.framework.simulation.RandomVariable;

import java.util.List;

import java.io.IOException;



/**

 * @author Xin Bai

 * @version 1.0

 */

public class SimulationWithTime implements Constants {



	public SimulationWithTime() throws InstantiationException,

			IllegalAccessException, IOException {



		ArrayList inputs = new ArrayList();

		for (int timeUnit = 0; timeUnit <= 10000; timeUnit = timeUnit +

		 500) {

//		for (int timeUnit = 20; timeUnit <= 100000; timeUnit = timeUnit + 5000) {

            // timeUnit cannot be zero

			SimulationInputParameters sim = new SimulationInputParameters();

			sim.setParameter(TIMEUNIT, timeUnit);

			sim.setParameter(DURATIONVALUE, 1600);

			sim.setStartTime(0);

			sim.setStopTime(50);

			inputs.add(sim);

		}

		// runs the simulation

		List outputs = Simulation.simulationSet(inputs,

				BrokerageSimulation.class, BrokerageSimulationContext.class);

		// create plots

		ArrayList revenue = new ArrayList();

		ArrayList allocatedResource = new ArrayList();

		ArrayList admittedUsers = new ArrayList();

		ArrayList resourceUtility = new ArrayList();

		ArrayList resourceSatisfaction = new ArrayList();

		ArrayList userUtility = new ArrayList();

		ArrayList userSatisfaction = new ArrayList();

		ArrayList userAdmissionRate = new ArrayList();

		ArrayList allocationBalance = new ArrayList();

		ArrayList allocationPercentageBalance = new ArrayList();



		String sValues[] = SVALUE.split(" ");

		int sList[] = new int[sValues.length];

		for (int i = 0; i < sList.length; i++) {

			sList[i] = Integer.parseInt(sValues[i]);

		}



		String uValues[] = UVALUE.split(" ");

		double uList[] = new double[uValues.length];

		for (int i = 0; i < uList.length; i++) {

			uList[i] = Double.parseDouble(uValues[i]);

		}



		for (int i = 0; i < sList.length; i++) {

			int s = sList[i];



			for (int j = 0; j < uList.length; j++) {

				double u = uList[j];



				String whichLabel;

				String label;



				if (sList.length > 1) {

					whichLabel = Integer.toString(s);

					label = "s-level = " + s;

					if (s == 0) {

						label = "random";

					}

				} else {

					whichLabel = Double.toString(u);

					label = "u-level = " + u;

				}



				revenue.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG, REVENUE + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				allocatedResource.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG,

						ALLOCATEDRESOURCE + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				admittedUsers.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG, ADMITTEDUSERS + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				resourceUtility.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG, RESOURCEUTILITY + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				resourceSatisfaction.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG, RESOURCESATISFACTION

								+ whichLabel, RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				userUtility.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG, USERUTILITY + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				userSatisfaction.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG, USERSATISFACTION + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				userAdmissionRate.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG,

						USERADMISSIONRATE + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				allocationBalance.add(new PlotLineDescription(TIMEUNIT,

						RandomVariable.TYPE_AVG,

						ALLOCATIONBALANCE + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



				allocationPercentageBalance.add(new PlotLineDescription(

						TIMEUNIT, RandomVariable.TYPE_AVG,

						ALLOCATIONPERCENTAGEBALANCE + whichLabel,

						RandomVariable.TYPE_AVG,

						RandomVariable.TYPE_CONFINT_RADIUS, label));



			}

		}



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/Revenue.m", revenue, outputs, "",

						"Time", "Average Revenue of Resource Providers Per Unit Time");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/AllocatedResource.m",

						allocatedResource, outputs, "", "Time",

						"Average Allocated Resources of Providers (reu)");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/AdmittedUsers.m", admittedUsers,

						outputs, "", "Time",

						"Average Admitted Users of Resource Providers Per Unit Time");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/ResourceUtility.m",

						resourceUtility, outputs, "", "Time",

						"Average Resource Utility of Providers");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/ResourceSatisfaction.m",

						resourceSatisfaction, outputs, "", "Time",

						"Average Resource Satisfaction of Providers");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/UserUtility.m", userUtility,

						outputs, "", "Time", "Average Utility of Users");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/UserSatisfaction.m",

						userSatisfaction, outputs, "", "Time",

						"Average Satisfaction of Users");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/UserAdmissionRate.m",

						userAdmissionRate, outputs, "", "Time",

						"User Admission Rate of the System");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/AllocationBalance.m",

						allocationBalance, outputs, "", "Time",

						"Resource Load Balance Degree of the System");



		SimulationHelper

				.generateErrorBarPlotFromPlotDescriptionListWithTitleLabel(

						"data/gridbrokerage/AllocationPercentageBalance.m",

						allocationPercentageBalance, outputs, "", "Time",

						"Resource Load Percentage Balance Degree of the System");



	}

}

