package yaes.application.marketgrid.brokers;

import yaes.application.marketgrid.Broker;
import yaes.application.marketgrid.money.Bank;
import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.application.marketgrid.subprotocols.SubProtBroker;
import yaes.application.marketgrid.subprotocols.SubProtMatchmakerQuery;
import yaes.application.marketgrid.subprotocols.SubProtProviderExecution;
import yaes.application.marketgrid.subprotocols.SubProtProviderPriceQuery;
import yaes.framework.agent.ExtendedMessage;
import yaes.ui.text.TextUi;

public class BrokerConversation {

    enum BrokerConversationState {
        NO_ACTION, STATE_RESOURCE_REQUEST, STATE_RESOURCE_MATCH, STATE_PRICE_REQUEST,
        STATE_PRICE_OFFER
    }

    private String id = null;
    private Broker broker;
    private String currentClient;
    private BrokerConversationState state;

    private AllocationDescriptor requestedResource;
    private SubProtProviderPriceQuery subProtProviderPriceQuery;
    private SubProtMatchmakerQuery subProtMatchmakerQuery;
    private SubProtBroker subProtBroker;
    private String offeredProvider;

    public BrokerConversation(Broker broker,
            ExtendedMessage message) {
        this.broker = broker;
        this.id = message.getConversation();
        subProtProviderPriceQuery = new SubProtProviderPriceQuery(broker);
        subProtMatchmakerQuery = new SubProtMatchmakerQuery(broker);
        subProtBroker = new SubProtBroker(broker);

        //@FIXME expect only conversations that client initiates. Otherwise ignore !!!
        if ( ! message.getSubprotocol().equals(SubProtBroker.BROKER_SUBPROTOCOL)){
          state = BrokerConversationState.NO_ACTION;
          return;
        }

        this.currentClient = message.getSender();

        if (message.getValue(SubProtProviderPriceQuery.PARAM_ASKING_PRICE) != null) {
          state = BrokerConversationState.STATE_PRICE_REQUEST;
          requestedResource = (AllocationDescriptor) message
                    .getValue(SubProtProviderPriceQuery.PARAM_ASKING_PRICE);
        }
        else if (message.getValue(SubProtProviderExecution.PARAM_REQUESTED_RESOURCES) != null) {
            state = BrokerConversationState.STATE_RESOURCE_REQUEST;
            requestedResource = (AllocationDescriptor) message
                    .getValue(SubProtProviderExecution.PARAM_REQUESTED_RESOURCES);
        }

    }

    public boolean isFinished() {
        return state == BrokerConversationState.NO_ACTION;
    }

    public void action(ExtendedMessage in_message) {

        TextUi.println("ACTION at BrokerConversationState " + id);
        switch (state) {
        case STATE_RESOURCE_REQUEST: {
            ExtendedMessage out_message = subProtMatchmakerQuery
                    .createQueryMessage("MatchMaker_0", id, requestedResource);
            broker.send(out_message);
            state = BrokerConversationState.NO_ACTION;
            break;
        }
        case STATE_RESOURCE_MATCH: {
            offeredProvider = (String) in_message
                    .getValue(SubProtMatchmakerQuery.PARAM_OFFERED_PROVIDER);
            state = BrokerConversationState.STATE_PRICE_REQUEST;
            break;
        }
        case STATE_PRICE_REQUEST: {
            ExtendedMessage out_message = subProtProviderPriceQuery
                    .createQueryMessage(offeredProvider, id, requestedResource);
            broker.send(out_message);
            state = BrokerConversationState.STATE_PRICE_OFFER;
            break;
        }
        case STATE_PRICE_OFFER: {
            double priceOffer = ((Double) in_message
                .getValue(SubProtProviderPriceQuery.PARAM_ASKING_PRICE)).doubleValue();
            Offer offer = new Offer();
            offer.allocationDescriptor = requestedResource.getCopy(false);
            offer.allocationDescriptor.expectedUnitPrice = priceOffer;
            offer.provider = offeredProvider;
            ExtendedMessage reply = subProtBroker
                  .createResourceAnswerMessage(currentClient, id, offer);
            broker.send(reply);
            state = BrokerConversationState.NO_ACTION;
            break;
        }
        }

    }
}
