package yaes.application.marketgrid.providers;

import java.util.ArrayList;
import java.util.Collection;

import yaes.application.marketgrid.clients.Task;
import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.application.marketgrid.subprotocols.SubProtProviderExecution;
import yaes.application.marketgrid.subprotocols.SubProtProviderPriceQuery;
import yaes.application.marketgrid.subprotocols.SubProtProviderWholeSale;
import yaes.application.marketgrid.subprotocols.SubProtProviderCommited;
import yaes.framework.agent.ExtendedMessage;
import yaes.ui.text.TextUi;

public class ProviderExecuteConversations
       extends ProviderConversation {

    public enum ProviderExecuteStates {
        STATE_NOACTION,
        STATE_FREE_STARTING_EXECUTION, STATE_FREE_IN_EXECUTION,
        STATE_COMMITED_STARTING_EXECUTION, STATE_COMMITED_IN_EXECUTION
    };


    private ProviderExecuteStates state;
    private AllocationDescriptor requestedResource;
    private SubProtProviderExecution subProtProviderExecution;
    private SubProtProviderCommited subProtProviderCommited;

    private String id = null;
    private MultiProvider provider;
    private String currentClient;

    public ProviderExecuteConversations(
                      MultiProvider provider, ExtendedMessage message,
                      ProviderExecuteStates state) {

        this.provider = provider;
        this.id = message.getConversation();
        this.currentClient = message.getSender();
        this.state = state;

        subProtProviderCommited = new SubProtProviderCommited(provider);
        subProtProviderExecution  = new SubProtProviderExecution(provider);
    }

    public boolean isFinished(){
      return (state == ProviderExecuteStates.STATE_NOACTION);
    }

    public void action(ExtendedMessage in_message) {

        TextUi.println("ACTION at ProviderExecuteConversations " + provider.getName()+ " state "+state);
        switch (state) {
        case STATE_NOACTION: {
            break;
        }
        case STATE_FREE_STARTING_EXECUTION: {

            Task task = (Task) in_message.getValue(SubProtProviderExecution.PARAM_TASK);
            if(provider.addTaskToRunFree(task, id, currentClient))
              state = ProviderExecuteStates.STATE_FREE_IN_EXECUTION;
            else{
              ExtendedMessage reply = subProtProviderExecution
                          .createExecutionRejectMessage(currentClient, id);
              provider.send(reply);
              state = ProviderExecuteStates.STATE_NOACTION;
            }
            break;
        }
        case STATE_FREE_IN_EXECUTION: {
          if(in_message.getSender().equals(provider.getName())){
            // send the termination message
            ExtendedMessage message = subProtProviderExecution
                    .createExecutionFinishedMessage(currentClient, id);
            provider.send(message);
            state = ProviderExecuteStates.STATE_NOACTION;
            TextUi.println("TASK DONE at ProviderExecuteConversations " + provider.getName()+ " state "+state);
            break;
          }
          else{
            // reject the incoming messages
            ExtendedMessage reply = subProtProviderExecution
                        .createExecutionRejectMessage(in_message.getSender(), id);
            provider.send(reply);
            // loop until TaskRunner notifies
          }
          break;
        }
        case STATE_COMMITED_STARTING_EXECUTION: {

            Task task = (Task) in_message.getValue(SubProtProviderCommited.PARAM_TASK);
            if(provider.addTaskToRunCommited(task, id, currentClient))
              state = ProviderExecuteStates.STATE_COMMITED_IN_EXECUTION;
            else{
              ExtendedMessage reply = subProtProviderCommited
                          .createExecutionRejectMessage(currentClient, id);
              provider.send(reply);
              state = ProviderExecuteStates.STATE_NOACTION;
            }
            break;
        }
        case STATE_COMMITED_IN_EXECUTION: {
          if(in_message.getSender().equals(provider.getName())){
            // send the termination message
            ExtendedMessage message = subProtProviderCommited
                    .createExecutionFinishedMessage(currentClient, id);
            provider.send(message);
            state = ProviderExecuteStates.STATE_NOACTION;
            TextUi.println("TASK DONE at ProviderExecuteConversations " + provider.getName()+ " state "+state);
            break;
          }
          else{
            // reject the incoming messages
            ExtendedMessage reply = subProtProviderCommited
                        .createExecutionRejectMessage(in_message.getSender(), id);
            provider.send(reply);
            // loop until TaskRunner notifies
          }
          break;
        }
        }
    }
}
