package yaes.application.marketgrid.providers;

import java.util.ArrayList;
import java.util.Collection;

import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.application.marketgrid.subprotocols.SubProtProviderExecution;
import yaes.application.marketgrid.subprotocols.SubProtProviderPriceQuery;
import yaes.application.marketgrid.subprotocols.SubProtProviderWholeSale;
import yaes.application.marketgrid.subprotocols.SubProtProviderCommited;
import yaes.framework.agent.ExtendedMessage;
import yaes.ui.text.TextUi;

public class ProviderSalesConversations
       extends ProviderConversation {

    public enum ProviderSalesStates {
        STATE_NOACTION,
        STATE_RETAIL_PRICE_OFFER,
        STATE_WHOLESALE_PRICE_OFFER, STATE_WHOLESALE_SALE_REQUEST
    };

    private ProviderSalesStates state;
    private AllocationDescriptor requestedResource;
    private SubProtProviderPriceQuery subProtProviderPriceQuery;
    private SubProtProviderWholeSale subProtProviderWholeSale;

    private String id = null;
    private MultiProvider provider;
    private String currentClient;

    public boolean isFinished(){
      return (state == ProviderSalesStates.STATE_NOACTION);
    }

    public ProviderSalesConversations(MultiProvider provider,
            ExtendedMessage message, ProviderSalesStates state) {

        this.provider = provider;
        this.id = message.getConversation();
        this.currentClient = message.getSender();
        this.state = state;
        subProtProviderWholeSale  = new SubProtProviderWholeSale(provider);
        subProtProviderPriceQuery = new SubProtProviderPriceQuery(provider);
    }

    public void action(ExtendedMessage in_message) {

        TextUi.println("ACTION at ProviderSalesConversations " + provider.getName() +" state "+state);
        switch (state) {
        case STATE_NOACTION: {
            break;
        }
        case STATE_RETAIL_PRICE_OFFER: {
            requestedResource = (AllocationDescriptor)in_message.
                  getValue(SubProtProviderPriceQuery.PARAM_ASKING_PRICE);
            // make a price offer
            double quantity = requestedResource.quantity;
            if(quantity == 0.0)
              quantity = requestedResource.quantityUpperBound;

            if(quantity == 0.0)
              quantity = requestedResource.quantityLowerBound;

            ExtendedMessage out_message = subProtProviderPriceQuery
                    .createPriceOfferMessage(currentClient,
                            id, provider.getPriceOffer(quantity));
            provider.send(out_message);
            state = ProviderSalesStates.STATE_NOACTION;
            break;
        }
        case STATE_WHOLESALE_PRICE_OFFER: {

            requestedResource = (AllocationDescriptor)in_message.
                  getValue(SubProtProviderWholeSale.PARAM_REQUESTED_RESOURCES);
            ExtendedMessage out_message;

            Collection<AllocationDescriptor> offers = provider.getWholeSaleOffer(
                                          requestedResource);
            if(offers != null)
              out_message = subProtProviderWholeSale.
                                          createPriceOfferMessage(currentClient, id, offers);
            else
              out_message = subProtProviderWholeSale.createOfferRejectMessage(
                                              currentClient, id, requestedResource);
            provider.send(out_message);
            state = ProviderSalesStates.STATE_NOACTION;
            break;
        }
        case STATE_WHOLESALE_SALE_REQUEST: {
            requestedResource = (AllocationDescriptor)in_message.
                  getValue(SubProtProviderWholeSale.PARAM_REQUESTED_RESOURCES);
            ExtendedMessage out_message;
            if(provider.allocateCommited(requestedResource, currentClient))
              out_message = subProtProviderWholeSale.createSaleAcceptMessage(
                                              currentClient, id, requestedResource);
            else
              out_message = subProtProviderWholeSale.createOfferRejectMessage(
                                              currentClient, id, requestedResource);
            provider.send(out_message);
            state = ProviderSalesStates.STATE_NOACTION;
            break;
        }
        }
    }
}
