package yaes.application.marketgrid.providers;

import yaes.application.marketgrid.clients.Task;
import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.application.marketgrid.resource.IResource;
import yaes.application.marketgrid.subprotocols.SubProtProviderExecution;

import yaes.framework.agent.ExtendedMessage;
import yaes.ui.text.TextUi;

public class TaskRunner{

    Task task;
    MultiProvider provider;
    String conversationId;
    int timeToFinish;
    String client;
    boolean isRunning;
    IResource resource;
    boolean isCommited;

    public TaskRunner(Task task, MultiProvider provider, String conversationId,
                      String client, IResource resource, boolean isCommited){
      this.task = task;
      this.provider = provider;
      this.conversationId = conversationId;
      this.client = client;
      this.resource = resource;
      this.isCommited = isCommited;
      timeToFinish = (int)task.getRealExecutionTime(resource);
      isRunning = true;

      // allocate physical
      AllocationDescriptor required = task.getRequiredResources();
      // try to allocate upper bound first. If fails try then lower bound
      // it should not fail, since check passed prior to
      if(resource.allocate(required.quantityUpperBound, client, false) == 0.0){
        if(resource.allocate(required.quantityLowerBound, client, false) == 0.0){
          throw new Error("Could not allocate physical resource "+resource.getResourceId()+
             " with quantity " + required.quantityLowerBound +" for "+client);
        }
        else{
          // executing at lower bound
          required.quantity = required.quantityLowerBound;
        }
      }
      else{
        // executing at upper bound
        required.quantity = required.quantityUpperBound;
      }
    }

    public void action() {
            // progress on the current task
      timeToFinish--;

      if (timeToFinish == 0) {
        SubProtProviderExecution subProtProviderExecution  = new SubProtProviderExecution(provider);
        ExtendedMessage out_message = subProtProviderExecution.createExecutionFinishedMessage(
                                              provider.getName(), conversationId);
        provider.receiveMessage(out_message);
        isRunning = false;
        // release physical
        resource.release(task.getRequiredResources().quantity, client);
        // release bussines allocation
        provider.releaseResource(conversationId);
        // charge dues
        if(isCommited)
          provider.chargeCommitedResource(client, task.getRequiredResources());
        else
          provider.chargeResource(client, task.getRequiredResources());
      }
    }

    public boolean hasTerminated(){
      return ! isRunning;
    }
}
