package yaes.application.marketgrid.resource;

import java.util.ArrayList;
import java.util.Collection;

/**
 *  Describes negotiated resource in terms set of distingushing
 *  parameters, time frame and intensity range. It is used as a
 *  request or offer so that negotiating sides can make decision
 */

public class AllocationDescriptor{

  /**
   *    Set of resource's distinguishing parameters required to fulfill request
   *    (e.g. VOLTAGE = 110, CURRENT = 50 A)
   */
  public Collection<AllocationRequestParameter> resourceParameters;

  /**
   *    Timestamp when use of resource is expected to start
   *    if period of usage is subject of negotiation
   */
  //public Calendar startUsageTime;
  public double startUsageTime;

  /**
   *    Timestamp when use of resource is expected to end
   *    if period of usage is subject of negotiation
   */
  //public Calendar endUsageTime;
  public double endUsageTime;

  /**
   *    Expected price for use of resource in given period
   */
  public double expectedUnitPrice;

  /**
   *    If resource is delivered in batches, quantity and its range have to be 
   *    expressed as whole number multiples of batch size. Value 0.0 means
   *    that resource can be delivered in fractions of unit
   */
  public double quantityBatchSize;

  /**
   *    Expected quantity available for use of resource in given period
   *    if single value is subject of negotiation
   */
  public double quantity;
  /**
   *    Expected minimum quantity available for use of resource in given period
   *    if range of values is subject of negotiation
   */
  public double quantityLowerBound;

  /**
   *    Expected maximum quantity available for use of resource in given period
   *    if range of values is subject of negotiation
   */
  public double quantityUpperBound;

  /**
   *    Copies instance of data
   *    @param deep true : requests copying of collections' by elements' values
   *                false : collections will be copied as references
   *    @return new instance with copied values
   */
  public AllocationDescriptor getCopy(boolean deep){

    AllocationDescriptor copy = new AllocationDescriptor();
    if(deep){
      copy.resourceParameters = new ArrayList<AllocationRequestParameter>();
      for(AllocationRequestParameter param : resourceParameters){
        copy.resourceParameters.add(param);
      }
    }
    else
      copy.resourceParameters = resourceParameters;
    copy.startUsageTime = startUsageTime;
    copy.endUsageTime = endUsageTime;
    copy.expectedUnitPrice = expectedUnitPrice;
    copy.quantity = quantity;
    copy.quantityLowerBound = quantityLowerBound;
    copy.quantityUpperBound = quantityUpperBound;
    return copy;
  }
}

