package yaes.application.marketgrid.resource.simple;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import yaes.application.marketgrid.resource.IPricingRule;
import yaes.application.marketgrid.resource.IResource;
import yaes.application.marketgrid.resource.ResourceParameter;
import yaes.application.marketgrid.resource.UsageChunk;

public class SimpleResource
       implements IResource{

  private String resourceType;
  private String resourceId;
  private double resourceTotalQuantity;
  private double resourceAvailableQuantity;

  private Collection<IPricingRule> pricingRules;
  private Collection<ResourceParameter> resourceParameters;
  private Collection<IResource> resourceBindings;
  private ArrayList<UsageChunk> allocationChunks;
  private boolean busy = false;
  private Object synchronizationToken = new Object();

  public SimpleResource(){
    init();
  }

  public void setTotalQuantity(double resourceTotalQuantity){
    this.resourceTotalQuantity = resourceTotalQuantity;
    resourceAvailableQuantity = resourceTotalQuantity;
  }

  public double getAvailableQuantity(){
    return resourceAvailableQuantity;
  }

  public double allocate(double requestedQuantity, String user, boolean fragment){

    synchronized(synchronizationToken){
      while(busy){
        try{
        synchronizationToken.wait();
        }
        catch(InterruptedException ie){
        }
      }
      busy = true;
      double ret_value = allocate_sync(requestedQuantity, user, fragment);
      busy = false;
      synchronizationToken.notify();
      return ret_value;
    }
  }

  private double allocate_sync(double requestedQuantity, String user, boolean fragment){

    if(resourceAvailableQuantity == 0.0)
      return 0.0;

      // enforce non-fragmentation of resource
    if(! fragment &&
      resourceAvailableQuantity - requestedQuantity < 0.0)
      return 0.0;

    UsageChunk chunk = new UsageChunk();
    allocationChunks.add(chunk);
    chunk.user = user;
    if(resourceAvailableQuantity - requestedQuantity >= 0.0)
      chunk.quantity = requestedQuantity;
    else
      chunk.quantity = resourceAvailableQuantity;
    resourceAvailableQuantity -= chunk.quantity;
    return chunk.quantity;
  }

  public UsageChunk allocate(UsageChunk requestedQuantityChunk, boolean fragment){
    UsageChunk chunk = new UsageChunk();
    chunk.quantity = allocate(requestedQuantityChunk.quantity, requestedQuantityChunk.user, fragment);
    return chunk;
  }

  public void release(double releaseQuantity, String user){

    synchronized(synchronizationToken){
      while(busy){
        try{
        synchronizationToken.wait();
        }
        catch(InterruptedException ie){
        }
      }
      busy = true;
      release_sync(releaseQuantity, user);
      busy = false;
      synchronizationToken.notify();
    }
  }

  private void release_sync(double releaseQuantity, String user){

    for(Iterator<UsageChunk> it=allocationChunks.iterator(); it.hasNext();){
      UsageChunk chunk = it.next();
      if(chunk.user == user &&
         chunk.quantity == releaseQuantity){
        resourceAvailableQuantity += chunk.quantity;
        it.remove();
        return;
      }
    }

    //  request and allocation chunks do not match, release chunks until request fulfilled
    double to_release = releaseQuantity;
    for(Iterator<UsageChunk> it=allocationChunks.iterator(); it.hasNext();){
      UsageChunk chunk = it.next();
      if(chunk.user == user){
        if(chunk.quantity <= to_release){
        to_release -= chunk.quantity;
        resourceAvailableQuantity += chunk.quantity;
        it.remove();
        if(to_release == 0.0)
          return;
        }
        else{
          chunk.quantity -= to_release;
          resourceAvailableQuantity += to_release;
          return;
        }
      }
    }
  }

  public void release(UsageChunk releaseQuantity){
    release(releaseQuantity.quantity, releaseQuantity.user);
  }

  public String getResourceType(){
    return resourceType;
  }

  public void setResourceType(String resourceType){
    this.resourceType = resourceType;
  }

  public String getResourceId(){
    return resourceId;
  }

  public void setResourceId(String resourceId){
    this.resourceId = resourceId;
  }

  public void setPricingPolicy(Collection<IPricingRule> pricingRules){
    this.pricingRules = pricingRules;
  }

  public Collection<IPricingRule> getPricingPolicy(){
    return pricingRules;
  }

  public Collection<ResourceParameter> getResourceParameters(){
    return resourceParameters;
  }

  public void setResourceParameters(Collection<ResourceParameter> resourceParameters){
    this.resourceParameters = resourceParameters;
  }

  public Collection<IResource> getResourceBindings(){
    return resourceBindings;
  }

  public void setResourceBindings(Collection<IResource> resourceBindings){
    this.resourceBindings = resourceBindings;
  }

  private void init(){
    allocationChunks = new ArrayList<UsageChunk>();
    resourceAvailableQuantity = resourceTotalQuantity;
  }

}
