package yaes.application.marketgrid.subprotocols;

import java.util.Collection;

import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.framework.agent.AbstractCommunicatingAgent;
import yaes.framework.agent.ExtendedMessage;

public class SubProtMatchmakerQuery extends AbstractSubprotocol {

	public static final String MSG_QUERY = "Query";
	public static final String MSG_ANSWER = "Answer";
	public static final String PARAM_REQUESTED_RESOURCES = "RequestedResources";
	public static final String PARAM_OFFERED_PROVIDER = "OfferedProvider";
	public static final String PARAM_MULTIPLE_PROVIDERS_OFFERED = "MultipleProvidersOffered";

	public SubProtMatchmakerQuery(AbstractCommunicatingAgent agent) {
		super("MatchMakerQuery", agent);
	}

    public ExtendedMessage createQueryMessage(String destination, String conversation,
        AllocationDescriptor allocationDescriptor) {
        ExtendedMessage message = createQueryMessage(destination, allocationDescriptor);
        message.setConversation(conversation);
        return message;
    }

    public ExtendedMessage createQueryMessage(String destination,
        AllocationDescriptor allocationDescriptor) {
      ExtendedMessage message = new ExtendedMessage(destination, agent
          .getName(), ExtendedMessage.Performative.REQUEST);
      message.setSubprotocol(getName());
      message.setSubprotocolMessage(MSG_QUERY);
      message.setValue(PARAM_REQUESTED_RESOURCES, allocationDescriptor);
      return message;
    }

    public ExtendedMessage createAnswerMessage(ExtendedMessage query, String conversation,
        String offeredProvider) {
      ExtendedMessage reply = new ExtendedMessage(query.getSender(), agent
          .getName(), ExtendedMessage.Performative.ANSWER);
      reply.setSubprotocol(getName());
      reply.setSubprotocolMessage(MSG_ANSWER);
      reply.setConversation(conversation);
      reply.setValue(PARAM_OFFERED_PROVIDER, offeredProvider);
      return reply;
    }

    public ExtendedMessage createAnswerMessage(ExtendedMessage query, String conversation,
        Collection<String> offeredProviders) {
      ExtendedMessage reply = new ExtendedMessage(query.getSender(), agent
          .getName(), ExtendedMessage.Performative.ANSWER);
      reply.setSubprotocol(getName());
      reply.setSubprotocolMessage(MSG_ANSWER);
      reply.setConversation(conversation);
      reply.setValue(PARAM_MULTIPLE_PROVIDERS_OFFERED, offeredProviders);
      return reply;
    }
}
