package yaes.application.marketgrid.subprotocols;

import java.util.Collection;

import yaes.application.marketgrid.resource.AllocationDescriptor;
import yaes.framework.agent.AbstractCommunicatingAgent;
import yaes.framework.agent.ExtendedMessage;

public class SubProtProviderWholeSale extends AbstractSubprotocol {

    static public final String PARAM_ASKING_PRICE = "AskingPriceWholeSale";
    static public final String PARAM_REQUESTED_RESOURCES = "RequestedResourcesWholeSale";

    static public final String MSG_PRICE_QUERY = "PriceQueryWholeSale";
    static public final String MSG_PRICE_OFFER = "PriceOfferWholeSale";
    static public final String MSG_OFFER_REJECT = "OfferRejectWholeSale";
    static public final String MSG_SALE_REQUEST = "SaleRequestWholeSale";
    static public final String MSG_SALE_ACCEPT = "SaleAcceptWholeSale";

    public SubProtProviderWholeSale(AbstractCommunicatingAgent agent) {
        super("ProviderWholeSale", agent);
    }

    public ExtendedMessage createQueryMessage(String destination, String conversation,
            AllocationDescriptor allocationDescriptor) {

        ExtendedMessage message = new ExtendedMessage(destination, agent
                .getName(), ExtendedMessage.Performative.REQUEST);
        message.setConversation(conversation);
        message.setValue(PARAM_ASKING_PRICE, allocationDescriptor);
        return message;
    }

    public ExtendedMessage createPriceOfferMessage(String destination, String conversation,
                                                  Collection<AllocationDescriptor> priceOffer) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(),
                                                          ExtendedMessage.Performative.ACCEPT);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_PRICE_OFFER);
        message.setConversation(conversation);
        message.setValue(SubProtProviderPriceQuery.PARAM_ASKING_PRICE, priceOffer);
        return message;
    }

    public ExtendedMessage createOfferRejectMessage(String destination, String conversation,
                                                  AllocationDescriptor descriptor) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(),
                ExtendedMessage.Performative.REJECT);
        message.setConversation(conversation);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_OFFER_REJECT);
        message.setValue(
                PARAM_REQUESTED_RESOURCES,
                descriptor);
        return message;
    }

    public ExtendedMessage createSaleRequestMessage(String destination, String conversation,
                                                              AllocationDescriptor descriptor) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(),
                ExtendedMessage.Performative.REQUEST);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_SALE_REQUEST);
        message.setValue(
                PARAM_REQUESTED_RESOURCES,
                descriptor);
        return message;
    }

    public ExtendedMessage createSaleAcceptMessage(String destination, String conversation,
                                                          AllocationDescriptor descriptor) {
        ExtendedMessage message = new ExtendedMessage(destination, agent.getName(),
                ExtendedMessage.Performative.ACCEPT);
        message.setConversation(conversation);
        message.setSubprotocol(getName());
        message.setSubprotocolMessage(MSG_SALE_ACCEPT);
        message.setValue(
                PARAM_REQUESTED_RESOURCES,
                descriptor);
        return message;
    }
}

