/*

 * Created on Sep 30, 2004

 *

 * TODO To change the template for this generated file go to

 * Window - Preferences - Java - Code Style - Code Templates

 */

package yaes.application.taskallocation;



import java.util.ArrayList;

import java.util.Collections;

import java.util.Iterator;

import java.util.List;

import umontreal.iro.lecuyer.probdist.PoissonDist;

import umontreal.iro.lecuyer.randvar.PoissonGen;

import umontreal.iro.lecuyer.rng.RandMrg;

import yaes.framework.simulation.IContext;

import yaes.framework.simulation.ISimulationCode;

import yaes.framework.simulation.SimulationInputParameters;

import yaes.framework.simulation.SimulationOutputParameters;



/**

 * @author Lotzi Boloni

 * 

 * TODO To change the template for this generated type comment go to Window -

 * Preferences - Java - Code Style - Code Templates

 */

public class DDTASimulationCode implements ISimulationCode, Constants {



    /*

     * (non-Javadoc)

     * 

     * @see yaes.application.taskallocation.ISimulationCode#setup(yaes.application.taskallocation.SimulationInputParameters)

     */



    PoissonGen gen;



    public void setup(SimulationInputParameters sip, IContext theContext) {

        gen = new PoissonGen(new RandMrg(), new PoissonDist(sip

                .getParameterInteger(TASKPERSEC)));

        GridContext context = (GridContext) theContext;

        // creating the nodes

        for (int i = 0; i != sip.getParameterInteger(NODECOUNT); i++) {

            GridNode node = new GridNode(i);

            context.nodes.add(node);

        }

        // now link them up

        for (int i = 0; i != sip.getParameterInteger(UPLINKS); i++) {

            createACycle(context.nodes);

        }

        // the start node

        context.startNode = (GridNode) context.nodes.get(0);

    }



    public void createACycle(List listOfNodes) {

        // create a copy of the list

        List copy = new ArrayList();

        copy.addAll(listOfNodes);

        GridNode firstNode = selectRandomNodeFromList(copy);

        copy.remove(firstNode);

        GridNode previousNode = firstNode;

        while (!copy.isEmpty()) {

            GridNode current = selectRandomNodeFromList(copy);

            copy.remove(current);

            previousNode.getUpstreamNodes().add(current);

            current.getDownstreamNodes().add(previousNode);

            previousNode = current;

        }

        // completing the cycle

        previousNode.getUpstreamNodes().add(firstNode);

        firstNode.getDownstreamNodes().add(previousNode);

    }



    /*

     * (non-Javadoc)

     * 

     * @see yaes.application.taskallocation.ISimulationCode#update(double,

     *      yaes.application.taskallocation.SimulationOutputParameters)

     */

    public void update(double time, SimulationInputParameters sip,

            SimulationOutputParameters sop, IContext theContext) {

        GridContext context = (GridContext) theContext;

        insertTasks(sip, context);

        double countBusy = 0;

        for (Iterator iter = context.nodes.iterator(); iter.hasNext();) {

            GridNode element = (GridNode) iter.next();

            element.update(time, sip, sop);

            if (element.isBusy()) {

                countBusy++;

            }

        }

        // update the variable

        sop.update(BUSY, countBusy / context.nodes.size());

    }



    /**

     * Insert the tasks through a single start node

     * 

     * @param sip

     * @param context

     */

    private void insertTasks(SimulationInputParameters sip, GridContext context) {

        // insert tasks at a single location

        int count;

        int prewalk = sip.getParameterInteger(PREWALKNUMBER);

        if (sip.getParameterInteger(TASKARRIVAL) == TASKARRIVAL_POISSON) {

            count = gen.nextInt();

        } else {

            count = sip.getParameterInteger(TASKPERSEC);

        }

        if (sip.getParameterInteger(TASKINSERTION) == TASKINSERTION_ONEPOINT) {

            for (int i = 0; i != count; i++) {

                context.startNode

                        .receiveTask(new GridTask(context.taskCount++, prewalk));

            }

        } else {

            for (int i = 0; i != count; i++) {

                GridNode node = selectRandomNodeFromList(context.nodes);

                node.receiveTask(new GridTask(context.taskCount++, prewalk));

            }

        }

    }

    

    public static GridNode selectRandomNodeFromList(List list) {

        int n = list.size();

        int nodeId = (int) (Math.random() * n);

        GridNode node = (GridNode) list.get(nodeId);

        return node;

    }



    /**

     * The fairness numbers are the 5% and 95% quotile numbers for the allocated 

     * tasks. It is obtained by sorting the nodes by allocated tasks, and picking the corresponding 

     * minimum, 5% quotile, 95% quotile and maximum numbers. 

     */

    public void extractFairnessNumbers(SimulationOutputParameters sop, GridContext context) {

        ArrayList toSort = new ArrayList();

        toSort.addAll(context.nodes);        

        Collections.sort(toSort);

        int len = toSort.size();

        int min = ((GridNode)toSort.get(0)).getFinishedTasks();

        sop.update(TASKS_PER_NODE_MIN, min);

        int max = ((GridNode)toSort.get(len-1)).getFinishedTasks();

        sop.update(TASKS_PER_NODE_MAX, max);

        int perc5 = ((GridNode)toSort.get((int)(0.05 * len))).getFinishedTasks();

        sop.update(TASKS_PER_NODE_PERC5, perc5);

        int perc95 = ((GridNode)toSort.get((int)(0.95 * len))).getFinishedTasks();

        sop.update(TASKS_PER_NODE_PERC95, perc95);

    }

    

    

    public String toString() {

        return "Directed Diffusion for Task Allocation";

    }



    /* (non-Javadoc)

     * @see yaes.framework.simulation.ISimulationCode#postprocess(yaes.framework.simulation.SimulationInputParameters, yaes.framework.simulation.SimulationOutputParameters, yaes.framework.simulation.IContext)

     */

    public void postprocess(SimulationInputParameters sip, SimulationOutputParameters sop, IContext theContext) {

        extractFairnessNumbers(sop, (GridContext)theContext);

    }

}