package yaes.application.uavaltclient.coalitionformation;



import java.io.IOException;



import yaes.application.uavaltclient.coalitionformation.timing.TimingException;

import yaes.application.uavaltclient.nav.Pos;

import yaes.ui.plot.SimulationPlot;



/**

 * This class is the test harness which is used to run coalition formation without the rest of 

 * the UAV simulation.

 * 

 * @author dejongp

 */

public class Main

{

  public static void main(String args[])

  {

		//origTest();

		

		plotTest();

  }

  

	public static void plotTest()

	{

		SimulationPlot s = new SimulationPlot();

		double xPoints[] = {10, 20, 30, 40};

		double res1[] = {10.0, 20, 30, 40};

		double res2[] = {50, 65, 15, 11};

		s.setXPoints(xPoints);

		s.addOnePlot("N=10", res1, "b-");

		s.addOnePlot("N=20", res2, "b-");

		

		s.setTitle("data");

		s.setXLabel("X AXIS");

		s.setYLabel("Y AXIS");

		

		try

		{

			// interpolates, no points shown

			s.generateMatlabPlotFile("C:\\MATLAB6p5\\work\\test.m");

		  

		  // interpolates with points shown

			s.generateMatlabPlotFileWithLegend("C:\\MATLAB6p5\\work\\test2.m");

			

			// not sure what this one does

			//s.generateMatlabRelativeDistancePlotFile("C:\\MATLAB6p5\\work\\test3.m");

		}

		catch (IOException e)

		{

				

		}

	}  

	

	// good test... shows the order you would expect (biggest speedup from pruning, followed by iterative deepening)

	private static void origTest()

	{

		//CoalitionFormation bf = new DepthFirstCoalitionFormation(false); //depth-first, pruning off

		//CoalitionFormation bf = new DepthFirstCoalitionFormation(true); //depth-first, pruning on

	

		//CoalitionFormation bf = new DFIDCoalitionFormation(false); //depth-first iterative deepening, pruning off

		//CoalitionFormation bf = new DFIDCoalitionFormation(true); //depth-first iterative deepening, pruning on

		

		CoalitionFormation bf = new GACoalitionFormation();

		

		Pos startingPos = new Pos(50.0, 0.0, 50.0);

		

		//MissionRepresentation(String name, double firepower, int numBombs, double visibility, double maxVelocity, double stealth)

		//bf.addMissionRepresentation(new MissionRepresentation("mission-all", 200.0, 50.0, 3.2, 50));

		

		bf.addMissionRepresentation(new MissionRepresentation("mission-75fp-75s-1b", 75.0, 1, 0.0, 0.0, 75.0, (Pos) startingPos.clone()));

		bf.addMissionRepresentation(new MissionRepresentation("mission-50fp-75s", 50.0, 0, 0.0, 0.0, 75.0, (Pos) startingPos.clone()));

		bf.addMissionRepresentation(new MissionRepresentation("mission-impossible", 50.0, 0, 0.0, 0.0, 100.0, (Pos) startingPos.clone()));



		bf.addMissionRepresentation(new MissionRepresentation("mission-20fp-20vis", 20.0, 0, 20.0, 0.0, 0.0, (Pos) startingPos.clone()));

		

		

		bf.addUAVRepresentation(new UAVRepresentation("name1", 25.0, 0, 65.0, 3.3, 2.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name2", 55.0, 0, 65.0, 3.3, 40.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name3", 33.3, 0, 10.0, 3.3, 3.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name4", 10.0, 0, 2.2, 3.3, 20.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name5", 1.1, 0, 30.0, 3.3, 10.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name6", 8.2, 0, 2.2, 3.3, 15.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name7-bomber", 1.1, 1, 11.0, 3.3, 26.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name8", 15.0, 0, 2.2, 3.3, 100.0, (Pos) startingPos.clone()));

		bf.addUAVRepresentation(new UAVRepresentation("name9", 15.0, 0, 20.2, 3.3, 100.0, (Pos) startingPos.clone()));



		

		

		for (int i=0; i<12; i++) // slows down dramatically after 20

		{

			bf.addUAVRepresentation(new UAVRepresentation("red-herring" + i, 0.1, 0, 2.2, 3.3, 4.4, (Pos) startingPos.clone()));

		}





		bf.formCoalitions();

		

		bf.display();

		try

		{

			System.out.println("Execution time: " + bf.getTotalTime().getElapsedTime() + " sec.");

		}

		catch(TimingException e)

		{

			System.out.println("ERROR, MUST STOP TIMER FIRST.");

		}

		System.out.println("Number combinations considered: " + bf.getNumCombosConsidered().toString());

		

		for (int i=0; i<bf.getCoalitions().size(); i++)

		{

			System.out.println("Coalition " + i + ": size " + ((Coalition) bf.getCoalitions().get(i)).getUAVRepresentations().size());

		}

	}	

	

	

}

