package yaes.application.uavaltclient.coalitionformation.ga;



import yaes.framework.algorithm.geneticalgorithm.IChromosome;



/**

 * Bitmap chromosomal representation which models one potential agent coalition / 

 * uav squadron.  An array representation is used, where the index identifies

 * a uav and the value is a boolean (true=uav is present in formation, false=

 * not present).

 * 

 * @author dejongp

 */

public class ChromosomeImpl implements IChromosome

{

	private boolean[] squadBitmap;

	

	/**

	 * Constructs a new Chromosome, randomly populated.

	 * 

	 * @param length number of agents / uavs in squadron and therefore represented in the chromosome

	 */

	public ChromosomeImpl(int length)

	{

		squadBitmap = new boolean[length];

		

		for (int i=0; i<length; i++)

		{

			squadBitmap[i] = Math.random() < 0.5;

		}

	}



  /**

   * Construct a new Chromosome using the supplied squad bitmap.

   * 

   * @param squadBitmap boolean array of agents/uavs representing those potentially present in a squadron

   */

  public ChromosomeImpl(boolean[] squadBitmap)

  {

  	this.squadBitmap = squadBitmap;

  }



  /**

   * @return

   */

  public boolean[] getSquadBitmap()

  {

    return squadBitmap;

  }



}

