package yaes.application.uavaltclient.coalitionformation.testharness;



import java.util.ArrayList;

import java.util.List;

import java.util.Random;



import yaes.application.uavaltclient.coalitionformation.UAVRepresentation;

import yaes.application.uavaltclient.nav.Pos;



/**

 * Pool of available UAVs for test harness.  Forming coalitions of current production uavs

 * would not be possible due to the huge range of capabilities.  Instead, 6 types of

 * fictional UAV are introduced.  Some of the uavs have several different configurations

 * on the same airframe, increasing the number of UAVs from which coalitions can be formed.

 * 

 * @author dejongp

 */

public class UAVPool

{

	public static final Pos STARTING_POS = new Pos(40.0, 0.0, 40.0);

	private static Random generator = new Random();

	

	//String name, double firepower, double visibility, double maxVelocity, double stealth



	//

	// large bomber

	//

	// Contains large payload bombs and very little defense.  Large, clumsy

	// and slow.  Usually escorted by fighters.

	//

	public static UAVRepresentation largeBomber1 = new UAVRepresentation("large bomber 1", 10, 4, 10, 2.5, 15, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation largeBomber2 = new UAVRepresentation("large bomber 2", 10, 4, 10, 2.5, 15, (Pos) STARTING_POS.clone()); 

	

	// a more stealthy large bomber variant

	public static UAVRepresentation largeBomber3 = new UAVRepresentation("large bomber 3 - stealthy variant", 5, 4, 10, 2.5, 25, (Pos) STARTING_POS.clone()); 

	

	//

	// large fighter

	//

	// This UAV is equipped with air-to-air munitions including missiles and 

	// machine gun.  It has a stealthy design, and contains radar-jamming devices, further 

	// lowering its stealth rating.  However, it is also quite large.

	//

	public static UAVRepresentation largeFighter1 = new UAVRepresentation("large fighter 1", 80, 0, 20, 3.5, 45, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation largeFighter2 = new UAVRepresentation("large fighter 2", 80, 0, 20, 3.5, 45, (Pos) STARTING_POS.clone());

	

	// large fighter variant with more firepower at the expense of stealth

	public static UAVRepresentation largeFighter3 = new UAVRepresentation("large fighter 3 - inc firepower variant", 90, 0, 20, 3.5, 35, (Pos) STARTING_POS.clone());

	

	//

	// large recon  

	//

	// This UAV is large and fast, but equipped only with sensors and no munitions.

	//

	public static UAVRepresentation largeRecon1 = new UAVRepresentation("large recon 1", 0, 0, 65, 3.7, 75, (Pos) STARTING_POS.clone()); 



  //

  // medium bomber

  //

  // This UAV uses its payload for air-to-ground missles or smart-bombs.

  //

	public static UAVRepresentation mediumBomber1 = new UAVRepresentation("medium bomber 1", 15, 2, 15, 2.7, 20, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation mediumBomber2 = new UAVRepresentation("medium bomber 2", 15, 2, 15, 2.7, 20, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation mediumBomber3 = new UAVRepresentation("medium bomber 3", 15, 2, 15, 2.7, 20, (Pos) STARTING_POS.clone()); 

  

  //

  // medium fighter

  // 

  // This UAV is primarily intended for use escorting bombers or other UAVs or 

  // fighting other aircraft.  It is agile and stealthy.

  // 

	public static UAVRepresentation mediumFighter1 = new UAVRepresentation("medium fighter 1", 55, 0, 15, 3.0, 60, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation mediumFighter2 = new UAVRepresentation("medium fighter 2", 55, 0, 15, 3.0, 60, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation mediumFighter3 = new UAVRepresentation("medium fighter 3", 55, 0, 15, 3.0, 60, (Pos) STARTING_POS.clone()); 



  // medium fighter variants with additional velocity

	public static UAVRepresentation mediumFighter4 = new UAVRepresentation("medium fighter 4 - inc vel variant", 50, 0, 15, 3.3, 60, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation mediumFighter5 = new UAVRepresentation("medium fighter 5 - inc vel variant", 50, 0, 15, 3.3, 60, (Pos) STARTING_POS.clone()); 



	//

	// small recon

	//

	// This UAV has an extremely small footprint and contains a camera and infrared

	// sensor. However, its range is quite limited.

	//

	public static UAVRepresentation smallRecon1 = new UAVRepresentation("small recon 1", 0, 0, 55, 3.4, 95, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation smallRecon2 = new UAVRepresentation("small recon 2", 0, 0, 55, 3.4, 95, (Pos) STARTING_POS.clone()); 

	public static UAVRepresentation smallRecon3 = new UAVRepresentation("small recon 3", 0, 0, 55, 3.4, 95, (Pos) STARTING_POS.clone()); 

	

	// a slower small recon variant with better visibility

	public static UAVRepresentation smallRecon4 = new UAVRepresentation("small recon 4 - inc visibility variant", 0, 0, 75, 2.9, 90, (Pos) STARTING_POS.clone());

	public static UAVRepresentation smallRecon5 = new UAVRepresentation("small recon 5 - inc visibility variant", 0, 0, 75, 2.9, 90, (Pos) STARTING_POS.clone());

	

	// an extremely fast recon variant

	public static UAVRepresentation smallRecon6 = new UAVRepresentation("small recon 6 - inc vel variant", 0, 0, 75, 3.4, 60, (Pos) STARTING_POS.clone());

 

 

	public static final List aircraft = new ArrayList();

	

	static

	{

		aircraft.add(smallRecon1);

		aircraft.add(smallRecon2);

		aircraft.add(smallRecon3);

		aircraft.add(smallRecon4);

		aircraft.add(smallRecon5);

		aircraft.add(smallRecon6);

				

		aircraft.add(mediumBomber1);

		aircraft.add(mediumBomber2);

		aircraft.add(mediumBomber3);

				

		aircraft.add(largeBomber1);

		aircraft.add(largeBomber2);

		aircraft.add(largeBomber3);

		

		aircraft.add(largeRecon1);

		

		aircraft.add(mediumFighter1);

		aircraft.add(mediumFighter2);

		aircraft.add(mediumFighter3);

		aircraft.add(mediumFighter4);

		aircraft.add(mediumFighter5);



		aircraft.add(largeFighter1);

		aircraft.add(largeFighter2);

		aircraft.add(largeFighter3);

	}  

  	

	/**

	 * Randomly select one of the uavs in the pool and return it.

	 * 

	 * @return randomly selected uav from pool

	 */

	public static UAVRepresentation getRandomUAV()

	{

		int acIndex = generator.nextInt(aircraft.size());

		return (UAVRepresentation) aircraft.get(acIndex);			

	}

	

	/**

	 * Returns a randomly sorted version of the aircraft pool

	 * 

	 * @return

	 */

	public static List getRandomPool(int size)

	{

		List list = new ArrayList();

		while (list.size() < size)

		{

			UAVRepresentation rep;

			

			do

			{

				rep = getRandomUAV();	

			}

			while (list.contains(rep));

			

			list.add(rep);

		}

		

		return list;

	}

	

}

