package yaes.application.uavaltclient.missionmgr;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import yaes.application.uavaltclient.naming.ExtendedName;

/**
 * Initial mission and squadron assignments for use during startup.  Instantiates all UAVs.
 * Reads data from initial_assignments.xml configuration file.
 * 
 * @author dejongp
 */
public class InitialAssignment
{
	private static final String FILENAME = "initial_assignment.xml";
	private static final String UAV_PACKAGE = "yaes.application.uavaltclient.agents";
	
	//String array of all uavs in system
	private String[] members = null;
	
  //maps uav names to squadronName for all uavs in system (including leader)
	private HashMap squadronUAVs = new HashMap();
	
  //maps squadronName to mission filename for this squadron 
	private HashMap missions = new HashMap();
	
	//arrayList of instantiated UAVs (will be used by Main's control loop)
	private ArrayList uavs = new ArrayList();
	
	private List extendedNames = new ArrayList(); // ExtendedName[]
	
	/**
	 * Create object using the xml configuration file.
	 */
	public InitialAssignment()
	{	
		// parse xml file
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		
		try
		{
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(FILENAME);
		  
			NodeList nl = doc.getElementsByTagName("assignments");
			for (int i = 0; i < nl.getLength(); i++)
			{
				Element assignmentsElement = (Element) nl.item(i);
         
				NodeList nl3 = assignmentsElement.getElementsByTagName("squadron");
				for (int j = 0; j < nl3.getLength(); j++)
				{
					Element squadElement = (Element) nl3.item(j);
					ExtendedName ename = new ExtendedName();
					extendedNames.add(ename);
					String squadName = ename.toString();
					missions.put(squadName, squadElement.getAttribute("mission"));
					
					NodeList nl5 = squadElement.getElementsByTagName("uav");
					for (int k = 0; k < nl5.getLength(); k++)
					{
						Element uavElement = (Element) nl5.item(k);						
						String name = uavElement.getAttribute("name");
						squadronUAVs.put(name, squadName);
						
						// instantiate UAV
						Class uavClass = Class.forName(UAV_PACKAGE + "." + uavElement.getAttribute("type"));
						Constructor uavConstructor = uavClass.getConstructor(
						  new Class[]{String.class, double.class, double.class, double.class, double.class});
						  
						uavs.add(uavConstructor.newInstance(
						  new Object[]{  
						    uavElement.getAttribute("name"),
						    new Double(uavElement.getAttribute("firepower")),
						    new Double(uavElement.getAttribute("visibility")),
						    new Double(uavElement.getAttribute("max_velocity")),
						    new Double(uavElement.getAttribute("stealth"))}));

					}
				}
			}
			

			members = new String[squadronUAVs.size()];
			Iterator iter = squadronUAVs.keySet().iterator();
			int i = 0;
			while (iter.hasNext() )
				members[i++] = (String) iter.next();  
						
		}
		catch (Exception e)
		{
			System.err.println("Error parsing xml initial_assignments file! " + e + ":" + e.getMessage() );		
		}
	}	

  /**
   * @return
   */
  public List getExtendedNames()
  {
    return extendedNames;
  }

  /**
   * @return
   */
  public String[] getMembers()
  {
    return members;
  }

  /**
   * @return
   */
  public HashMap getMissions()
  {
    return missions;
  }

  /**
   * @return
   */
  public HashMap getSquadronUAVs()
  {
    return squadronUAVs;
  }

  /**
   * @return
   */
  public ArrayList getUavs()
  {
    return uavs;
  }

}
