/*

 * Created on Sep 26, 2004

 *

 * To change the template for this generated file go to

 * Window - Preferences - Java - Code Generation - Code and Comments

 */

package yaes.framework.algorithm.pathplanning;



import java.util.ArrayList;

import java.util.Comparator;

import java.util.Iterator;

import yaes.framework.algorithm.search.IHeuristic;

import yaes.framework.algorithm.search.TreeNodeComparator;

import yaes.framework.algorithm.search.TreeSearchNode;

import yaes.framework.world.map.AbstractPathCost;

import yaes.framework.world.map.IMap;

import yaes.framework.world.map.Location;

import yaes.framework.world.map.PlannedPath;

import yaes.ui.text.TextUi;



/**

 * @author Lotzi Boloni

 *

 * To change the template for this generated type comment go to

 * Window - Preferences - Java - Code Generation - Code and Comments

 */

public class AStarPP extends AbstractTreePathPlanner {



    protected TreeSearchNode currentBest;

    protected double currentBestCost;

    private AbstractPathCost pathCost;

    private IHeuristic heuristic;

    private boolean returnFirst = false;



    /**

     * @param problemDescription

     * @param problemSpace

     */

    public AStarPP(PlannedPath problemDescription, IMap problemSpace,

            AbstractPathCost pathCost, IHeuristic heuristic) {

        super(problemDescription, problemSpace);

        this.pathCost = pathCost;

        this.heuristic = heuristic;

        currentBest = null;

        currentBestCost = Double.MAX_VALUE;

    }



    /* (non-Javadoc)

     * @see simulation.algorithms.pathplanning.AbstractTreePathPlanner#initFringe()

     */

    protected void initFringe() {

        //Comparator c1 = new DestinationComparator(thePath.getDestination());

        Comparator c2 = new TreeNodeComparator();

        //Comparator c2 = new AStarComparator(thePath.getDestination(), pathCost, heuristic);

        theFringe = new AStarFringe(c2, pathCost, thePath.getDestination(), heuristic);

    }



    /**

     * Should be generalizable for all optimizing searches.

     */

    public boolean planPath(PlannedPath path, IMap map) {

        TextUi.println("A* path planning from " + path.getSource() + " --> " + path.getDestination());

        // check the feasibility of the source and destination points

        if (getAccessibilityChecker().isAccessible(map, path.getSource()) == false) {

        	throw new Error("Source point " + path.getSource() + " is not accessible");

        }

        if (getAccessibilityChecker().isAccessible(map, path.getDestination()) == false) {

        	throw new Error("Destination point " + path.getDestination() + " is not accessible");

        }

        long startTime = System.currentTimeMillis();

        super.planPath(path, map);

        if (currentBest == null) {

            TextUi.println("No solution found for: " + path.getSource() + " -->" + path.getDestination());

            return false;

        }

        // readout the optimal solution

        if (!returnFirst) {

            ArrayList list = (ArrayList) readOutSolution(currentBest);

            for (Iterator iter = list.iterator(); iter.hasNext();) {

                Location element = (Location) iter.next();

                thePath.addLocation(element);

            }

        }

        double cost = pathCost.cost(path);

        TextUi.println("Best path found with cost=" + cost + "\ttime = "

                + ((System.currentTimeMillis() - startTime) / 1000)

                + " seconds");

        return true;

    }



    protected boolean isSolution(TreeSearchNode node) {

        if (super.isSolution(node)) {

            // evaluate the cost

            double cost = AStarFringe.getTreeNodeCost(pathCost, node);

            TextUi.println("Path found with cost=" + cost);

            if (cost < currentBestCost) {

                TextUi.println("Solution found, current cost is:" + cost);

                currentBest = node;

                currentBestCost = cost;

                ((AStarFringe) theFringe).setIgnoreBorder(currentBestCost);

            }

            if (returnFirst) {

                return true;

            }

        }

        // always return false, because we need to continue searching

        return false;

    }



    /**

     * @return Returns the returnFirst.

     */

    public boolean isReturnFirst() {

        return returnFirst;

    }



    /**

     * @param returnFirst The returnFirst to set.

     */

    public void setReturnFirst(boolean returnFirst) {

        this.returnFirst = returnFirst;

    }

}